@@ -214,6 +214,15 @@ const char *cookie_value(const char *zPName, const char *zDefault){
   return i<cookies.nParam ? cookies.aParam[i].zPValue : zDefault;
 }
 
+/* Return the number of characters of hex in the prefix to the
+** given string.
+*/
+static int hex_prefix_length(const char *z){
+  int i;
+  for(i=0; fossil_isXdigit(z[i]); i++){}
+  return i;
+}
+
 /*
 ** WEBPAGE: cookies
 **
@@ -231,6 +240,7 @@ void cookie_page(void){
   int nCookie = 0;
   const char *zName = 0;
   const char *zValue = 0;
+  const char *zLoginCookie = login_cookie_name();
   int isQP = 0;
   int bFDSonly = strstr(g.zPath, "fdscookie")!=0;
   cookie_parse();
@@ -256,11 +266,28 @@ void cookie_page(void){
     @ <input type="submit" name="%h(zDel)" value="Delete">
     if( fossil_strcmp(zName, DISPLAY_SETTINGS_COOKIE)==0  && cookies.nParam>0 ){
       int j;
+      @ <p>This cookie remembers your Fossil display preferences.
       @ <ul>
       for(j=0; j<cookies.nParam; j++){
         @ <li>%h(cookies.aParam[j].zPName): "%h(cookies.aParam[j].zPValue)"
       }
       @ </ul>
+    }else
+    if( fossil_strcmp(zName, zLoginCookie)==0 ){
+      @ <p>This is your login cookie.  If you delete this cookie, you will
+      @ be logged out.
+    }else
+    if( fossil_strncmp(zName, "fossil-", 7)==0
+     && strlen(zName)==23
+     && hex_prefix_length(&zName[7])==16
+     && hex_prefix_length(zValue)>24
+    ){
+      @ <p>This appears to be a login cookie for another Fossil repository
+      @ in the same website.
+    }
+    else {
+      @ <p>This cookie was not generated by Fossil.  It might be something
+      @ from another program on the same website.
     }
     fossil_free(zDel);
   }
