@@ -61,6 +61,7 @@ struct ForumThread {
   ForumPost *pDisplay;   /* Entries in display order */
   ForumPost *pTail;      /* Last on the display list */
   int mxIndent;          /* Maximum indentation level */
+  int nArtifact;         /* Number of forum artifacts in this thread */
 };
 #endif /* INTERFACE */
 
@@ -111,8 +112,13 @@ static int forumpost_head_rid(int rid){
 ** If bCheckIrt is true then p's thread in-response-to parents are
 ** checked (recursively) for closure, else only p is checked.
 */
-static int forumpost_is_closed(ForumPost *p, int bCheckIrt){
-  while(p){
+static int forumpost_is_closed(
+  ForumThread *pThread,          /* Thread that the post is a member of */
+  ForumPost *p,                  /* the forum post */
+  int bCheckIrt                  /* True to check In-Reply-To posts */
+){
+  int mx = pThread->nArtifact+1;
+  while( p && (mx--)>0 ){
     if( p->pEditHead ) p = p->pEditHead;
     if( p->iClosed || !bCheckIrt ) return p->iClosed;
     p = p->pIrt;
@@ -411,6 +417,7 @@ static ForumThread *forumthread_create(int froot, int computeHierarchy){
       pThread->pLast->pNext = pPost;
     }
     pThread->pLast = pPost;
+    pThread->nArtifact++;
 
     /* Find the in-reply-to post.  Default to the topic post if the replied-to
     ** post cannot be found. */
@@ -522,6 +529,7 @@ void forumthread_cmd(void){
   fossil_print("fpid  = %d\n", fpid);
   fossil_print("froot = %d\n", froot);
   pThread = forumthread_create(froot, 1);
+  fossil_print("count = %d\n", pThread->nArtifact);
   fossil_print("Chronological:\n");
   fossil_print(
 /* 0         1         2         3         4         5         6         7    */
@@ -567,6 +575,7 @@ void forumthreadhashlist(void){
   ForumThread *pThread;
   ForumPost *p;
   char *fuuid;
+  Stmt q;
 
   login_check_credentials();
   if( !g.perm.Admin ){
@@ -601,6 +610,23 @@ void forumthreadhashlist(void){
   }
   forumthread_delete(pThread);
   @ </pre>
+  @ <hr>
+  @ <h2>Related FORUMPOST Table Content</h2>
+  @ <table border="1" cellpadding="4" cellspacing="0">
+  @ <tr><th>fpid<th>froot<th>fprev<th>firt<th>fmtime
+  db_prepare(&q, "SELECT fpid, froot, fprev, firt, datetime(fmtime)"
+                 "  FROM forumpost"
+                 " WHERE froot=%d"
+                 " ORDER BY fmtime", froot);
+  while( db_step(&q)==SQLITE_ROW ){
+    @ <tr><td>%d(db_column_int(&q,0))\
+    @ <td>%d(db_column_int(&q,1))\
+    @ <td>%d(db_column_int(&q,2))\
+    @ <td>%d(db_column_int(&q,3))\
+    @ <td>%h(db_column_text(&q,4))</tr>
+  }
+  @ </table>
+  db_finalize(&q);
   style_finish_page();
 }
 
@@ -727,6 +753,7 @@ static char *forum_post_display_name(ForumPost *p, Manifest *pManifest){
 ** Display a single post in a forum thread.
 */
 static void forum_display_post(
+  ForumThread *pThread, /* The thread that this post is a member of */
   ForumPost *p,         /* Forum post to display */
   int iIndentScale,     /* Indent scale factor */
   int bRaw,             /* True to omit the border */
@@ -749,10 +776,10 @@ static void forum_display_post(
   /* Get the manifest for the post.  Abort if not found (e.g. shunned). */
   pManifest = manifest_get(p->fpid, CFTYPE_FORUM, 0);
   if( !pManifest ) return;
-  iClosed = forumpost_is_closed(p, 1);
+  iClosed = forumpost_is_closed(pThread, p, 1);
   /* When not in raw mode, create the border around the post. */
   if( !bRaw ){
-    /* Open the <div> enclosing the post.  Set the class string to mark the post
+    /* Open the <div> enclosing the post. Set the class string to mark the post
     ** as selected and/or obsolete. */
     iIndent = (p->pEditHead ? p->pEditHead->nIndent : p->nIndent)-1;
     @ <div id='forum%d(p->fpid)' class='forumTime\
@@ -1029,7 +1056,7 @@ static void forum_display_thread(
   /* Display the appropriate subset of posts in sequence. */
   while( p ){
     /* Display the post. */
-    forum_display_post(p, iIndentScale, mode==FD_RAW,
+    forum_display_post(pThread, p, iIndentScale, mode==FD_RAW,
         bUnf, bHist, p==pSelect, zQuery);
 
     /* Advance to the next post in the thread. */
@@ -1399,8 +1426,9 @@ static void forum_post_widget(
   }
   @ %z(href("%R/markup_help"))Markup style</a>:
   mimetype_option_menu(zMimetype, "mimetype");
-  @ <br><textarea aria-label="Content:" name="content" class="wikiedit" \
-  @ cols="80" rows="25" wrap="virtual">%h(zContent)</textarea><br>
+  @ <div class="forum-editor-widget">
+  @ <textarea aria-label="Content:" name="content" class="wikiedit" \
+  @ cols="80" rows="25" wrap="virtual">%h(zContent)</textarea></div>
 }
 
 /*
@@ -1522,7 +1550,7 @@ static void forum_render_debug_options(void){
     @ <br><label><input type="checkbox" name="showqp" %s(PCK("showqp"))> \
     @ Show query parameters</label>
     @ <br><label><input type="checkbox" name="fpsilent" %s(PCK("fpsilent"))> \
-    @ Do not sent notification emails</label>
+    @ Do not send notification emails</label>
     @ </div>
   }
 }
@@ -1762,6 +1790,21 @@ void forumedit_page(void){
   style_finish_page();
 }
 
+/*
+** SETTING: forum-close-policy    boolean default=off
+** If true, forum moderators may close/re-open forum posts, and reply
+** to closed posts. If false, only administrators may do so. Note that
+** this only affects the forum web UI, not post-closing tags which
+** arrive via the command-line or from synchronization with a remote.
+*/
+/*
+** SETTING: forum-title          width=20 default=Forum
+** This is the name or "title" of the Forum for this repository.  The
+** default is just "Forum".  But in some setups, admins might want to
+** change it to "Developer Forum" or "User Forum" or whatever other name
+** seems more appropriate for the particular usage.
+*/
+
 /*
 ** WEBPAGE: setup_forum
 **
@@ -1769,9 +1812,9 @@ void forumedit_page(void){
 */
 void forum_setup(void){
   /* boolean config settings specific to the forum. */
-  const char * zSettingsBool[] = {
-  "forum-close-policy",
-  NULL /* sentinel entry */
+  static const char *azForumSettings[] =  {
+    "forum-close-policy",
+    "forum-title",
   };
 
   login_check_credentials();
@@ -1790,10 +1833,8 @@ void forum_setup(void){
   }
 
   @ <h2>Supervisors</h2>
-  @ <p>Users with capabilities 's', 'a', or '6'.</p>
   {
     Stmt q = empty_Stmt;
-    int nRows = 0;
     db_prepare(&q, "SELECT uid, login, cap FROM user "
                    "WHERE cap GLOB '*[as6]*' ORDER BY login");
     @ <table class='bordered'>
@@ -1803,7 +1844,6 @@ void forum_setup(void){
       const int iUid = db_column_int(&q, 0);
       const char *zUser = db_column_text(&q, 1);
       const char *zCap = db_column_text(&q, 2);
-      ++nRows;
       @ <tr>
       @ <td><a href='%R/setup_uedit?id=%d(iUid)'>%h(zUser)</a></td>
       @ <td>(%h(zCap))</td>
@@ -1811,20 +1851,17 @@ void forum_setup(void){
     }
     db_finalize(&q);
     @</tbody></table>
-    if( 0==nRows ){
-      @ No supervisors
-    }else{
-      @ %d(nRows) supervisor(s)
-    }
   }
 
   @ <h2>Moderators</h2>
-  @ <p>Users with capability '5'.</p>
-  {
+  if( db_int(0, "SELECT count(*) FROM user "
+                " WHERE cap GLOB '*5*' AND cap NOT GLOB '*[as6]*'")==0 ){
+      @ <p>No non-supervisor moderators
+  }else{
     Stmt q = empty_Stmt;
-    int nRows = 0;
     db_prepare(&q, "SELECT uid, login, cap FROM user "
-               "WHERE cap GLOB '*5*' ORDER BY login");
+               "WHERE cap GLOB '*5*' AND cap NOT GLOB '*[as6]*'"
+               " ORDER BY login");
     @ <table class='bordered'>
     @ <thead><tr><th>User</th><th>Capabilities</th></tr></thead>
     @ <tbody>
@@ -1832,7 +1869,6 @@ void forum_setup(void){
       const int iUid = db_column_int(&q, 0);
       const char *zUser = db_column_text(&q, 1);
       const char *zCap = db_column_text(&q, 2);
-      ++nRows;
       @ <tr>
       @ <td><a href='%R/setup_uedit?id=%d(iUid)'>%h(zUser)</a></td>
       @ <td>(%h(zCap))</td>
@@ -1840,39 +1876,55 @@ void forum_setup(void){
     }
     db_finalize(&q);
     @ </tbody></table>
-    if( 0==nRows ){
-      @ No non-supervisor moderators
-    }else{
-      @ %d(nRows) moderator(s)
-    }
   }
 
   @ <h2>Settings</h2>
-  @ <p>Configuration settings specific to the forum.</p>
   if( P("submit") && cgi_csrf_safe(2) ){
     int i = 0;
-    const char *zSetting;
     db_begin_transaction();
-    while( (zSetting = zSettingsBool[i++]) ){
-      const char *z = P(zSetting);
-      if( !z || !z[0] ) z = "off";
-      db_set(zSetting/*works-like:"x"*/, z, 0);
+    for(i=0; i<ArraySize(azForumSettings); i++){
+      char zQP[4];
+      const char *z;
+      const Setting *pSetting = setting_find(azForumSettings[i]);
+      if( pSetting==0 ) continue;
+      zQP[0] = 'a'+i;
+      zQP[1] = zQP[0];
+      zQP[2] = 0;
+      z = P(zQP);
+      if( z==0 || z[0]==0 ) continue;
+      db_set(pSetting->name/*works-like:"x"*/, z, 0);
     }
     db_end_transaction(0);
     @ <p><em>Settings saved.</em></p>
   }
   {
     int i = 0;
-    const char *zSetting;
     @ <form action="%R/setup_forum" method="post">
     login_insert_csrf_secret();
     @ <table class='forum-settings-list'><tbody>
-    while( (zSetting = zSettingsBool[i++]) ){
-      @ <tr><td>
-      onoff_attribute("", zSetting, zSetting/*works-like:"x"*/, 0, 0);
-      @ </td><td>
-      @ <a href='%R/help?cmd=%h(zSetting)'>%h(zSetting)</a>
-      @ </td></tr>
+    for(i=0; i<ArraySize(azForumSettings); i++){
+      char zQP[4];
+      const Setting *pSetting = setting_find(azForumSettings[i]);
+      if( pSetting==0 ) continue;
+      zQP[0] = 'a'+i;
+      zQP[1] = zQP[0];
+      zQP[2] = 0;
+      if( pSetting->width==0 ){
+        /* Boolean setting */
+        @ <tr><td align="right">
+        @ <a href='%R/help?cmd=%h(pSetting->name)'>%h(pSetting->name)</a>:
+        @ </td><td>
+        onoff_attribute("", zQP, pSetting->name/*works-like:"x"*/, 0, 0);
+        @ </td></tr>
+      }else{
+        /* Text value setting */
+        @ <tr><td align="right">
+        @ <a href='%R/help?cmd=%h(pSetting->name)'>%h(pSetting->name)</a>:
+        @ </td><td>
+        entry_attribute("", 25, pSetting->name, zQP/*works-like:""*/,
+                        pSetting->def, 0);
+        @ </td></tr>
+      }   
     }
     @ </tbody></table>
     @ <input type='submit' name='submit' value='Apply changes'>
@@ -1911,7 +1963,8 @@ void forum_main_page(void){
   }
   cgi_check_for_malice();
   style_set_current_feature("forum");
-  style_header( "%s", isSearch ? "Forum Search Results" : "Forum" );
+  style_header("%s%s", db_get("forum-title","Forum"), 
+                       isSearch ? " Search Results" : "");
   style_submenu_element("Timeline", "%R/timeline?ss=v&y=f&vfx");
   if( g.perm.WrForum ){
     style_submenu_element("New Thread","%R/forumnew");
