@@ -19,12 +19,12 @@
       };
     },
     remove: function(e){
-      if(e.forEach){
+      if(e?.forEach){
         e.forEach(
-          (x)=>x.parentNode.removeChild(x)
+          (x)=>x?.parentNode?.removeChild(x)
         );
       }else{
-        e.parentNode.removeChild(e);
+        e?.parentNode?.removeChild(e);
       }
       return e;
     },
@@ -63,7 +63,7 @@
     }
     return str ? str.split(f.rx) : [str];
   };
-  
+
   dom.div = dom.createElemFactory('div');
   dom.p = dom.createElemFactory('p');
   dom.code = dom.createElemFactory('code');
@@ -89,7 +89,9 @@
       if(forElem instanceof HTMLElement){
         forElem = this.attr(forElem, 'id');
       }
-      dom.attr(rc, 'for', forElem);
+      if(forElem){
+        dom.attr(rc, 'for', forElem);
+      }
     }
     if(text) this.append(rc, text);
     return rc;
@@ -551,7 +553,7 @@
       old.parentNode.removeChild(old);
     }
   };
-  dom.replaceNode.counter = 0;        
+  dom.replaceNode.counter = 0;
   /**
      Two args == getter: (e,key), returns value
 
