@@ -61,7 +61,7 @@ static int sslNoCertVerify = 0;  /* Do not verify SSL certs */
 /* This is a self-signed cert in the PEM format that can be used when
 ** no other certs are available.
 */
-static const char sslSelfCert[] = 
+static const char sslSelfCert[] =
 "-----BEGIN CERTIFICATE-----\n"
 "MIIDMTCCAhkCFGrDmuJkkzWERP/ITBvzwwI2lv0TMA0GCSqGSIb3DQEBCwUAMFQx\n"
 "CzAJBgNVBAYTAlVTMQswCQYDVQQIDAJOQzESMBAGA1UEBwwJQ2hhcmxvdHRlMRMw\n"
@@ -85,7 +85,7 @@ static const char sslSelfCert[] =
 
 /* This is the private-key corresponding to the cert above
 */
-static const char sslSelfPKey[] = 
+static const char sslSelfPKey[] =
 "-----BEGIN PRIVATE KEY-----\n"
 "MIIEvAIBADANBgkqhkiG9w0BAQEFAASCBKYwggSiAgEAAoIBAQCCbTU26GRQHQqL\n"
 "q7vyZ0OxpAxmgfAKCxt6eIz+jBi2ZM/CB5vVXWVh2+SkSiWEA3UZiUqXxZlzmS/C\n"
@@ -208,7 +208,7 @@ static int ssl_client_cert_callback(SSL *ssl, X509 **x509, EVP_PKEY **pkey){
 /*
 ** Convert an OpenSSL ASN1_TIME to an ISO8601 timestamp.
 **
-** Per RFC 5280, ASN1 timestamps in X.509 certificates must 
+** Per RFC 5280, ASN1 timestamps in X.509 certificates must
 ** be in UTC (Zulu timezone) with no fractional seconds.
 **
 ** If showUtc==1, add " UTC" at the end of the returned string. This is
@@ -310,6 +310,25 @@ static void ssl_global_init_client(void){
       fossil_fatal("Cannot load CA root certificates from %s", zFile);
     }
 
+/* Enable OpenSSL to use the Windows system ROOT certificate store to search for
+** certificates missing in the file and directory trust stores already loaded by
+** `SSL_CTX_load_verify_locations()'.
+** This feature was introduced with OpenSSL 3.2.0, and may be enabled by default
+** for future versions of OpenSSL, and explicit initialization may be redundant.
+** NOTE TO HACKERS TWEAKING THEIR OPENSSL CONFIGURATION:
+** The following OpenSSL configuration options must not be used for this feature
+** to be available: `no-autoalginit', `no-winstore'. The Fossil makefiles do not
+** currently set these options when building OpenSSL for Windows. */
+#if defined(_WIN32)
+#if OPENSSL_VERSION_NUMBER >= 0x030200000
+    if( SSLeay()!=0x30500000  /* Don't use for 3.5.0 due to a bug */
+     && SSL_CTX_load_verify_store(sslCtx, "org.openssl.winstore:")==0
+    ){
+      fossil_print("NOTICE: Failed to load the Windows root certificates.\n");
+    }
+#endif /* OPENSSL_VERSION_NUMBER >= 0x030200000 */
+#endif /* _WIN32 */
+
     /* Load client SSL identity, preferring the filename specified on the
     ** command line */
     if( g.zSSLIdentity!=0 ){
@@ -388,12 +407,10 @@ static int establish_proxy_tunnel(UrlData *pUrlData, BIO *bio){
     bbuf = blob_buffer(&reply);
     len = blob_size(&reply);
     while(end < len) {
-      if(bbuf[end] == '\r') {
-        if(len - end < 4) {
-          /* need more data */
-          break;
-        }
-        if(memcmp(&bbuf[end], "\r\n\r\n", 4) == 0) {
+      if( bbuf[end]=='\n' ) {
+        if( (end+1<len && bbuf[end+1]=='\n')
+         || (end+2<len && bbuf[end+1]=='\r' && bbuf[end+2]=='\n')
+        ){
           done = 1;
           break;
         }
@@ -414,7 +431,7 @@ static int establish_proxy_tunnel(UrlData *pUrlData, BIO *bio){
 ** real server or a man-in-the-middle imposter.
 */
 void ssl_disable_cert_verification(void){
-  sslNoCertVerify = 1;  
+  sslNoCertVerify = 1;
 }
 
 /*
@@ -438,7 +455,14 @@ int ssl_open_client(UrlData *pUrlData){
     int rc;
     char *connStr = mprintf("%s:%d", g.url.name, pUrlData->port);
     BIO *sBio = BIO_new_connect(connStr);
-    free(connStr);
+    if( g.fIPv4 ){
+#ifdef BIO_FAMILY_IPV4
+      BIO_set_conn_ip_family(sBio, BIO_FAMILY_IPV4);
+#else
+      fossil_warning("The --ipv4 option is not supported in this build\n");
+#endif
+    }
+    fossil_free(connStr);
     if( BIO_do_connect(sBio)<=0 ){
       ssl_set_errmsg("SSL: cannot connect to proxy %s:%d (%s)",
             pUrlData->name, pUrlData->port,
@@ -490,7 +514,14 @@ int ssl_open_client(UrlData *pUrlData){
   if( !pUrlData->useProxy ){
     char *connStr = mprintf("%s:%d", pUrlData->name, pUrlData->port);
     BIO_set_conn_hostname(iBio, connStr);
-    free(connStr);
+    fossil_free(connStr);
+    if( g.fIPv4 ){
+#ifdef BIO_FAMILY_IPV4
+      BIO_set_conn_ip_family(iBio, BIO_FAMILY_IPV4);
+#else
+      fossil_warning("The --ipv4 option is not supported in this build\n");
+#endif
+    }
     if( BIO_do_connect(iBio)<=0 ){
       ssl_set_errmsg("SSL: cannot connect to host %s:%d (%s)",
          pUrlData->name, pUrlData->port,
@@ -567,12 +598,12 @@ int ssl_open_client(UrlData *pUrlData){
                  ssl_asn1time_to_iso8601(X509_get_notAfter(cert), 1));
       BIO_printf(mem, "\n  sha256:    %s", zHash);
       desclen = BIO_get_mem_data(mem, &desc);
-  
+
       prompt = mprintf("Unable to verify SSL cert from %s\n%.*s\n"
           "accept this cert and continue (y/N/fingerprint)? ",
           pUrlData->name, desclen, desc);
       BIO_free(mem);
-  
+
       prompt_user(prompt, &ans);
       free(prompt);
       cReply = blob_str(&ans)[0];
@@ -897,8 +928,8 @@ static void trust_location_usable(const char *zPath, const char **pzStore){
 #endif /* FOSSIL_ENABLE_SSL */
 
 /*
-** COMMAND: tls-config*
-** COMMAND: ssl-config
+** COMMAND: tls-config*                       abbrv-subcom
+** COMMAND: ssl-config                        abbrv-subcom
 **
 ** Usage: %fossil ssl-config [SUBCOMMAND] [OPTIONS...] [ARGS...]
 **
@@ -908,16 +939,16 @@ static void trust_location_usable(const char *zPath, const char **pzStore){
 **
 ** Sub-commands:
 **
-**   remove-exception DOMAINS    Remove TLS cert exceptions for the domains
-**                               listed.  Or remove them all if the --all
-**                               option is specified.
+**    remove-exception DOMAINS    Remove TLS cert exceptions for the domains
+**                                listed.  Or remove them all if the --all
+**                                option is specified.
 **
-**   scrub ?--force?             Remove all SSL configuration data from the
-**                               repository. Use --force to omit the
-**                               confirmation.
+**    scrub ?--force?             Remove all SSL configuration data from the
+**                                repository. Use --force to omit the
+**                                confirmation.
 **
-**   show ?-v?                   Show the TLS configuration. Add -v to see
-**                               additional explanation
+**    show ?-v?                   Show the TLS configuration. Add -v to see
+**                                additional explanation
 */
 void test_tlsconfig_info(void){
   const char *zCmd;
@@ -972,8 +1003,8 @@ void test_tlsconfig_info(void){
       );
     }
 #else
-    fossil_print("OpenSSL-version:      %s  (0x%09x)\n",
-         SSLeay_version(SSLEAY_VERSION), OPENSSL_VERSION_NUMBER);
+    fossil_print("OpenSSL-version:      %s  (0x%09llx)\n",
+         SSLeay_version(SSLEAY_VERSION), (unsigned long long)SSLeay());
     if( verbose ){
       fossil_print("\n"
          "  The version of the OpenSSL library being used\n"
@@ -1033,6 +1064,20 @@ void test_tlsconfig_info(void){
       );
     }
 
+#if defined(_WIN32)
+    fossil_print("  OpenSSL-winstore:   %s\n",
+         (SSLeay()>=0x30200000 && SSLeay()!=0x30500000) ? "Yes" : "No");
+    if( verbose ){
+      fossil_print("\n"
+         "    OpenSSL 3.2.0, or newer, but not version 3.5.0 due to a bug,\n"
+         "    are able to use the root certificates managed by the Windows\n"
+         "    operating system. The installed root certificates are listed\n"
+         "    by the command:\n\n"
+         "        certutil -store \"ROOT\"\n\n"
+      );
+    }
+#endif /* _WIN32 */
+
     if( zUsed==0 ) zUsed = "";
     fossil_print("  Trust store used:   %s\n", zUsed);
     if( verbose ){
@@ -1187,9 +1232,9 @@ wellknown_notfound:
 ** freed by the caller.
 */
 char *fossil_openssl_version(void){
-#if defined(FOSSIL_ENABLE_SSL) 
+#if defined(FOSSIL_ENABLE_SSL)
   return mprintf("%s (0x%09x)\n",
-         SSLeay_version(SSLEAY_VERSION), OPENSSL_VERSION_NUMBER);
+         SSLeay_version(SSLEAY_VERSION), (sqlite3_uint64)SSLeay());
 #else
   return mprintf("none");
 #endif
