@@ -75,22 +75,35 @@ static const struct JsonConfigProperty {
   char const * name;
   int groupMask;
 } JsonConfigProperties[] = {
-{ "css",                    CONFIGSET_CSS },
+{ "css",                    CONFIGSET_CSS  },
 { "header",                 CONFIGSET_SKIN },
+{ "mainmenu",               CONFIGSET_SKIN },
 { "footer",                 CONFIGSET_SKIN },
 { "details",                CONFIGSET_SKIN },
+{ "js",                     CONFIGSET_SKIN },
+{ "default-skin",           CONFIGSET_SKIN },
 { "logo-mimetype",          CONFIGSET_SKIN },
 { "logo-image",             CONFIGSET_SKIN },
 { "background-mimetype",    CONFIGSET_SKIN },
 { "background-image",       CONFIGSET_SKIN },
 { "icon-mimetype",          CONFIGSET_SKIN },
 { "icon-image",             CONFIGSET_SKIN },
-{ "timeline-block-markup",  CONFIGSET_SKIN },
+{ "timeline-date-format",   CONFIGSET_SKIN },
+{ "timeline-default-style", CONFIGSET_SKIN },
+{ "timeline-dwelltime",     CONFIGSET_SKIN },
+{ "timeline-closetime",     CONFIGSET_SKIN },
+{ "timeline-hard-newlines", CONFIGSET_SKIN },
 { "timeline-max-comment",   CONFIGSET_SKIN },
 { "timeline-plaintext",     CONFIGSET_SKIN },
+{ "timeline-truncate-at-blank", CONFIGSET_SKIN },
+{ "timeline-tslink-info",   CONFIGSET_SKIN },
+{ "timeline-utc",           CONFIGSET_SKIN },
 { "adunit",                 CONFIGSET_SKIN },
 { "adunit-omit-if-admin",   CONFIGSET_SKIN },
 { "adunit-omit-if-user",    CONFIGSET_SKIN },
+{ "default-csp",            CONFIGSET_SKIN },
+{ "sitemap-extra",          CONFIGSET_SKIN },
+{ "safe-html",              CONFIGSET_SKIN },
 
 { "project-name",           CONFIGSET_PROJ },
 { "short-project-name",     CONFIGSET_PROJ },
@@ -106,6 +119,15 @@ static const struct JsonConfigProperty {
 { "encoding-glob",          CONFIGSET_PROJ },
 { "empty-dirs",             CONFIGSET_PROJ },
 { "dotfiles",               CONFIGSET_PROJ },
+{ "parent-project-code",    CONFIGSET_PROJ },
+{ "parent-project-name",    CONFIGSET_PROJ },
+{ "hash-policy",            CONFIGSET_PROJ },
+{ "comment-format",         CONFIGSET_PROJ },
+{ "mimetypes",              CONFIGSET_PROJ },
+{ "forbid-delta-manifests", CONFIGSET_PROJ },
+{ "mv-rm-files",            CONFIGSET_PROJ },
+
+{ "user-color-map",         CONFIGSET_USER },
 
 { "ticket-table",           CONFIGSET_TKT  },
 { "ticket-common",          CONFIGSET_TKT  },
