@@ -20,7 +20,7 @@
 ** and/or forum posts.
 **
 ** The search can be either a per-query "grep"-like search that scans
-** the entire corpus.  Or it can use the FTS4 search engine of SQLite.
+** the entire corpus.  Or it can use the FTS5 search engine of SQLite.
 ** The choice is an administrator configuration option.
 **
 ** The first option is referred to as "full-scan search".  The second
@@ -566,32 +566,42 @@ void search_sql_setup(sqlite3 *db){
 **
 ** COMMAND: search*
 **
-** Usage: %fossil search [-a|-all] [-n|-limit #] [-W|-width #] pattern...
+** Usage: %fossil search [OPTIONS] PATTERN...
 **
-** Search for timeline entries matching all words provided on the
-** command line. Whole-word matches scope more highly than partial
-** matches.
+** Search the repository for PATTERN and show matches.  Depending on
+** options and how the administrator has search configured for the
+** repository, the search can cover:
 **
-** Note:  The command only search the EVENT table.  So it will only
-** display check-in comments or other comments that appear on an
-** unaugmented timeline.  It does not search document text or forum
-** messages.
+**    *   check-in comments (-c)
+**    *   embedded documentation (--docs)
+**    *   forum posts (--forum)
+**    *   tickets (--tickets)
+**    *   tech notes (--technotes)
+**    *   wiki pages (--wiki)
+**    *   built-in fossil help text (-h)
+**    *   all of the above (-a)
 **
-** Outputs, by default, some top-N fraction of the results. The -all
-** option can be used to output all matches, regardless of their search
-** score.  The -limit option can be used to limit the number of entries
-** returned.  The -width option can be used to set the output width used
-** when printing matches.
+** Use options below to select the scope of the search.  The
+** default is check-in comments only (-c).
+**
+** Output is colorized if writing to a TTY and if the NO_COLOR environment
+** variable is not set.  Use the "--highlight 0" option to disable colorization
+** or use "--highlight 91" to force it on.  Change the argument to --highlight
+** to change the color.
 **
 ** Options:
-**     -a|--all          Output all matches, not just best matches
-**     --debug           Show additional debug content on --fts search
-**     --fts             Use the full-text search mechanism (testing only)
+**     -a|--all          Search everything
+**     -c|--checkins     Search checkin comments
+**     --docs            Search embedded documentation
+**     --forum           Search forum posts
+**     -h|--bi-help      Search built-in help
+**     --highlight N     Used VT100 color N for matching text.  0 means "off".
 **     -n|--limit N      Limit output to N matches
-**     --scope SCOPE     Scope of search.  Valid for --fts only.  One or
-**                       more of: all, c, d, e, f, t, w.  Defaults to all.
+**     --technotes       Search tech notes
+**     --tickets         Search tickets
 **     -W|--width WIDTH  Set display width to WIDTH columns, 0 for
-**                       unlimited. Defaults the terminal's width.
+**                       unlimited. Defaults to the terminal's width.
+**     --wiki            Search wiki
 */
 void search_cmd(void){
   Blob pattern;
@@ -599,14 +609,30 @@ void search_cmd(void){
   Blob sql = empty_blob;
   Stmt q;
   int iBest;
+  int srchFlags = 0;
+  int bFts = 1;          /* Use FTS search by default now */
   char fAll = NULL != find_option("all", "a", 0);
   const char *zLimit = find_option("limit","n",1);
+  const char *zScope = 0;
   const char *zWidth = find_option("width","W",1);
-  const char *zScope = find_option("scope",0,1);
-  int bDebug = find_option("debug",0,0)!=0;
+  int bDebug = find_option("debug",0,0)!=0;     /* Undocumented */
   int nLimit = zLimit ? atoi(zLimit) : -1000;
   int width;
-  int bFts = find_option("fts",0,0)!=0;
+  int nTty = 0;          /* VT100 highlight color for matching text */
+  const char *zHighlight = 0;
+  int bFlags = 0;        /* DB open flags */
+
+  nTty =  terminal_is_vt100();
+
+  /* Undocumented option to change highlight color */
+  zHighlight = find_option("highlight",0,1);
+  if( zHighlight ) nTty = atoi(zHighlight);
+
+  /* Undocumented option (legacy) */
+  zScope = find_option("scope",0,1);
+
+  if( find_option("fts",0,0)!=0 ) bFts = 1;      /* Undocumented legacy */
+  if( find_option("legacy",0,0)!=0 ) bFts = 0;   /* Undocumented */
 
   if( zWidth ){
     width = atoi(zWidth);
@@ -616,9 +642,54 @@ void search_cmd(void){
   }else{
     width = -1;
   }
+  if( zScope ){
+    for(i=0; zScope[i]; i++){
+      switch( zScope[i] ){
+        case 'a':  srchFlags = SRCH_ALL;       break;
+        case 'c':  srchFlags |= SRCH_CKIN;     break;
+        case 'd':  srchFlags |= SRCH_DOC;      break;
+        case 'e':  srchFlags |= SRCH_TECHNOTE; break;
+        case 'f':  srchFlags |= SRCH_FORUM;    break;
+        case 'h':  srchFlags |= SRCH_HELP;     break;
+        case 't':  srchFlags |= SRCH_TKT;      break;
+        case 'w':  srchFlags |= SRCH_WIKI;     break;
+      }
+    }
+    bFts = 1;
+  }
+  if( find_option("all","a",0) ){      srchFlags |= SRCH_ALL;      bFts = 1; }
+  if( find_option("bi-help","h",0) ){  srchFlags |= SRCH_HELP;     bFts = 1; }
+  if( find_option("checkins","c",0) ){ srchFlags |= SRCH_CKIN;     bFts = 1; }
+  if( find_option("docs",0,0) ){       srchFlags |= SRCH_DOC;      bFts = 1; }
+  if( find_option("forum",0,0) ){      srchFlags |= SRCH_FORUM;    bFts = 1; }
+  if( find_option("technotes",0,0) ){  srchFlags |= SRCH_TECHNOTE; bFts = 1; }
+  if( find_option("tickets",0,0) ){    srchFlags |= SRCH_TKT;      bFts = 1; }
+  if( find_option("wiki",0,0) ){       srchFlags |= SRCH_WIKI;     bFts = 1; }
 
-  db_find_and_open_repository(0, 0);
+  /* If no search objects are specified, default to "check-in comments" */
+  if( srchFlags==0 ) srchFlags = SRCH_CKIN;
+
+  if( srchFlags==SRCH_HELP ) bFlags = OPEN_OK_NOT_FOUND|OPEN_SUBSTITUTE;
+  db_find_and_open_repository(bFlags, 0);
+  verify_all_options();
   if( g.argc<3 ) return;
+  login_set_capabilities("s", 0);
+  if( search_restrict(srchFlags)==0 && (srchFlags & SRCH_HELP)==0 ){
+    const char *zC1 = 0, *zPlural = "s";
+    if( srchFlags & SRCH_TECHNOTE ){  zC1 = "technote"; }
+    if( srchFlags & SRCH_TKT ){       zC1 = "ticket";   }
+    if( srchFlags & SRCH_FORUM ){     zC1 = "forum";    zPlural = ""; }
+    if( srchFlags & SRCH_DOC ){       zC1 = "document"; }
+    if( srchFlags & SRCH_WIKI ){      zC1 = "wiki";     zPlural = ""; }
+    if( srchFlags & SRCH_CKIN ){      zC1 = "check-in"; }
+    fossil_print(
+      "Search of %s%s is disabled on this repository.\n"
+      "Enable using \"fossil fts-config enable %s\".\n",
+      zC1, zPlural, zC1
+    );
+    return;
+  }
+
   blob_init(&pattern, g.argv[2], -1);
   for(i=3; i<g.argc; i++){
     blob_appendf(&pattern, " %s", g.argv[i]);
@@ -628,24 +699,6 @@ void search_cmd(void){
     Blob com;
     Blob snip;
     const char *zPattern = blob_str(&pattern);
-    int srchFlags;
-    unsigned int j;
-    if( zScope==0 ){
-      srchFlags = SRCH_ALL;
-    }else{
-      srchFlags = 0;
-      for(i=0; zScope[i]; i++){
-        switch( zScope[i] ){
-          case 'a':  srchFlags = SRCH_ALL;  break;
-          case 'c':  srchFlags |= SRCH_CKIN;     break;
-          case 'd':  srchFlags |= SRCH_DOC;      break;
-          case 'e':  srchFlags |= SRCH_TECHNOTE; break;
-          case 'f':  srchFlags |= SRCH_FORUM;    break;
-          case 't':  srchFlags |= SRCH_TKT;      break;
-          case 'w':  srchFlags |= SRCH_WIKI;     break;
-        }
-      }
-    }
     search_sql_setup(g.db);
     add_content_sql_commands(g.db);
     db_multi_exec(
@@ -656,26 +709,28 @@ void search_cmd(void){
     }else{
       search_update_index(srchFlags);        /* Update the index */
       search_indexed(zPattern, srchFlags);   /* Indexed search */
+      if( srchFlags & SRCH_HELP ){
+        search_fullscan(zPattern, SRCH_HELP);
+      }
     }
     db_prepare(&q, "SELECT snip, label, score, id, date"
                    "  FROM x"
                    " ORDER BY score DESC, date DESC;");
     blob_init(&com, 0, 0);
     blob_init(&snip, 0, 0);
-    if( width<0 ) width = 80;
+    if( width<0 ) width = terminal_get_width(80);
     while( db_step(&q)==SQLITE_ROW ){
       const char *zSnippet = db_column_text(&q, 0);
       const char *zLabel = db_column_text(&q, 1);
       const char *zDate = db_column_text(&q, 4);
       const char *zScore = db_column_text(&q, 2);
       const char *zId = db_column_text(&q, 3);
+      char *zOrig;
       blob_appendf(&snip, "%s", zSnippet);
-      for(j=0; j<snip.nUsed; j++){
-        if( snip.aData[j]=='\n' ){
-          if( j>0 && snip.aData[j-1]=='\r' ) snip.aData[j-1] = ' ';
-          snip.aData[j] = ' ';
-        }
-      }
+      zOrig = blob_materialize(&snip);
+      blob_init(&snip, 0, 0);
+      html_to_plaintext(zOrig, &snip, (nTty?HTOT_VT100:0)|HTOT_FLOW|HTOT_TRIM);
+      fossil_free(zOrig);
       blob_appendf(&com, "%s\n%s\n%s", zLabel, blob_str(&snip), zDate);
       if( bDebug ){
         blob_appendf(&com," score: %s id: %s", zScore, zId);
@@ -733,24 +788,29 @@ void search_cmd(void){
 #define SRCH_WIKI     0x0008    /* Search over wiki */
 #define SRCH_TECHNOTE 0x0010    /* Search over tech notes */
 #define SRCH_FORUM    0x0020    /* Search over forum messages */
-#define SRCH_ALL      0x003f    /* Search over everything */
+#define SRCH_HELP     0x0040    /* Search built-in help (full-scan only) */
+#define SRCH_ALL      0x007f    /* Search over everything */
 #endif
 
 /*
 ** Remove bits from srchFlags which are disallowed by either the
 ** current server configuration or by user permissions.  Return
 ** the revised search flags mask.
+**
+** If bFlex is true, that means allow through the SRCH_HELP option
+** even if it is not explicitly enabled.
 */
 unsigned int search_restrict(unsigned int srchFlags){
   static unsigned int knownGood = 0;
   static unsigned int knownBad = 0;
   static const struct { unsigned m; const char *zKey; } aSetng[] = {
-     { SRCH_CKIN,     "search-ci"   },
-     { SRCH_DOC,      "search-doc"  },
-     { SRCH_TKT,      "search-tkt"  },
-     { SRCH_WIKI,     "search-wiki" },
+     { SRCH_CKIN,     "search-ci"       },
+     { SRCH_DOC,      "search-doc"      },
+     { SRCH_TKT,      "search-tkt"      },
+     { SRCH_WIKI,     "search-wiki"     },
      { SRCH_TECHNOTE, "search-technote" },
-     { SRCH_FORUM,    "search-forum" },
+     { SRCH_FORUM,    "search-forum"    },
+     { SRCH_HELP,     "search-help"     },
   };
   int i;
   if( g.perm.Read==0 )   srchFlags &= ~(SRCH_CKIN|SRCH_DOC|SRCH_TECHNOTE);
@@ -796,27 +856,37 @@ LOCAL void search_fullscan(
     char *zDocGlob = db_get("doc-glob","");
     char *zDocBr = db_get("doc-branch","trunk");
     if( zDocGlob && zDocGlob[0] && zDocBr && zDocBr[0] ){
+      Glob * pGlob = glob_create(zDocBr)
+        /* We're misusing a Glob as a list of comma-/space-delimited
+        ** tokens. We're not actually doing glob matches here. */;
+      int i;
       db_multi_exec(
         "CREATE VIRTUAL TABLE IF NOT EXISTS temp.foci USING files_of_checkin;"
       );
-      db_multi_exec(
-        "INSERT INTO x(label,url,score,id,date,snip)"
-        "  SELECT printf('Document: %%s',title('d',blob.rid,foci.filename)),"
-        "         printf('/doc/%T/%%s',foci.filename),"
-        "         search_score(),"
-        "         'd'||blob.rid,"
-        "         (SELECT datetime(event.mtime) FROM event"
-        "            WHERE objid=symbolic_name_to_rid('trunk')),"
-        "         search_snippet()"
-        "    FROM foci CROSS JOIN blob"
-        "   WHERE checkinID=symbolic_name_to_rid('trunk')"
-        "     AND blob.uuid=foci.uuid"
-        "     AND search_match(title('d',blob.rid,foci.filename),"
-        "                      body('d',blob.rid,foci.filename))"
-        "     AND %z",
-        zDocBr, glob_expr("foci.filename", zDocGlob)
-      );
+      for( i = 0; i < pGlob->nPattern; ++i ){
+        const char * zBranch = pGlob->azPattern[i];
+        db_multi_exec(
+          "INSERT INTO x(label,url,score,id,date,snip)"
+          "  SELECT printf('Document: %%s',title('d',blob.rid,foci.filename)),"
+          "         printf('/doc/%T/%%s',foci.filename),"
+          "         search_score(),"
+          "         'd'||blob.rid,"
+          "         (SELECT datetime(event.mtime) FROM event"
+          "            WHERE objid=symbolic_name_to_rid(%Q)),"
+          "         search_snippet()"
+          "    FROM foci CROSS JOIN blob"
+          "   WHERE checkinID=symbolic_name_to_rid(%Q)"
+          "     AND blob.uuid=foci.uuid"
+          "     AND search_match(title('d',blob.rid,foci.filename),"
+          "                      body('d',blob.rid,foci.filename))"
+          "     AND %z",
+          zBranch, zBranch, zBranch, glob_expr("foci.filename", zDocGlob)
+        );
+      }
+      glob_free(pGlob);
     }
+    fossil_free(zDocGlob);
+    fossil_free(zDocBr);
   }
   if( (srchFlags & SRCH_WIKI)!=0 ){
     db_multi_exec(
@@ -904,6 +974,27 @@ LOCAL void search_fullscan(
       "   WHERE search_match('',body('f',rid,NULL));"
     );
   }
+  if( (srchFlags & SRCH_HELP)!=0 ){
+    const char *zPrefix;
+    helptext_vtab_register(g.db);
+    if( srchFlags==SRCH_HELP ){
+      zPrefix = "The";
+    }else{
+      zPrefix = "Built-in help for the";
+    }
+    db_multi_exec(
+      "INSERT INTO x(label,url,score,id,snip)"
+      "  SELECT format('%q \"%%s\" %%s',name,type),"
+      "         '/help?cmd='||name,"
+      "         search_score(),"
+      "         'h'||rowid,"
+      "         search_snippet()"
+      "    FROM helptext"
+      "   WHERE search_match(format('the \"%%s\" %%s',name,type),"
+      "                      helptext.helptext);",
+      zPrefix
+    );
+  }
 }
 
 /*
@@ -978,6 +1069,29 @@ static void search_rank_sqlfunc(
 #endif
 }
 
+/*
+** Expects a search pattern string. Makes a copy of the string,
+** replaces all non-alphanum ASCII characters with a space, and
+** lower-cases all upper-case ASCII characters. The intent is to avoid
+** causing errors in FTS5 searches with inputs which contain AND, OR,
+** and symbols like #. The caller is responsible for passing the
+** result to fossil_free().
+*/
+char *search_simplify_pattern(const char * zPattern){
+  char *zPat = mprintf("%s",zPattern);
+  int i;
+  for(i=0; zPat[i]; i++){
+    if( (zPat[i]&0x80)==0 && !fossil_isalnum(zPat[i]) ) zPat[i] = ' ';
+    if( fossil_isupper(zPat[i]) ) zPat[i] = fossil_tolower(zPat[i]);
+  }
+  for(i--; i>=0 && zPat[i]==' '; i--){}
+  if( i<0 ){
+    fossil_free(zPat);
+    zPat = mprintf("\"\"");
+  }
+  return zPat;
+}
+
 /*
 ** When this routine is called, there already exists a table
 **
@@ -999,23 +1113,15 @@ LOCAL void search_indexed(
   unsigned int srchFlags      /* What to search over */
 ){
   Blob sql;
-  char *zPat = mprintf("%s",zPattern);
-  int i;
+  char *zPat;
   static const char *zSnippetCall;
   if( srchFlags==0 ) return;
   sqlite3_create_function(g.db, "rank", 1, SQLITE_UTF8|SQLITE_INNOCUOUS, 0,
      search_rank_sqlfunc, 0, 0);
-  for(i=0; zPat[i]; i++){
-    if( (zPat[i]&0x80)==0 && !fossil_isalnum(zPat[i]) ) zPat[i] = ' ';
-  }
-  for(i--; i>=0 && zPat[i]==' '; i--){}
-  if( i<0 ){
-    fossil_free(zPat);
-    zPat = mprintf("\"\"");
-  }
+  zPat = search_simplify_pattern(zPattern);
   blob_init(&sql, 0, 0);
   if( search_index_type(0)==4 ){
-    /* If this repo is still using the legacy FTS4 search index, then
+    /* If this repo is still using the legacy FTS5 search index, then
     ** the snippet() function is slightly different */
     zSnippetCall = "snippet(ftsidx,'<mark>','</mark>',' ... ',-1,35)";
   }else{
@@ -1045,6 +1151,7 @@ LOCAL void search_indexed(
        { SRCH_WIKI,     'w' },
        { SRCH_TECHNOTE, 'e' },
        { SRCH_FORUM,    'f' },
+       { SRCH_HELP,     'h' },
     };
     int i;
     for(i=0; i<count(aMask); i++){
@@ -1134,7 +1241,7 @@ int search_run_and_output(
   if( P("searchlimit")!=0 ){
     nLimit = atoi(P("searchlimit"));
   }
-  srchFlags = search_restrict(srchFlags);
+  srchFlags = search_restrict(srchFlags) | (srchFlags & SRCH_HELP);
   if( srchFlags==0 ) return 0;
   search_sql_setup(g.db);
   add_content_sql_commands(g.db);
@@ -1146,6 +1253,9 @@ int search_run_and_output(
   }else{
     search_update_index(srchFlags);        /* Update the index, if necessary */
     search_indexed(zPattern, srchFlags);   /* Indexed search */
+    if( srchFlags & SRCH_HELP ){
+      search_fullscan(zPattern, SRCH_HELP);
+    }
   }
   db_prepare(&q, "SELECT url, snip, label, score, id, substr(date,1,10)"
                  "  FROM x"
@@ -1200,7 +1310,7 @@ int search_run_and_output(
 **
 ** Return true if there are search results.
 */
-int search_screen(unsigned srchFlags, int mFlags){
+int search_screen(unsigned srchAllowed, int mFlags){
   const char *zType = 0;
   const char *zClass = 0;
   const char *zDisable1;
@@ -1208,16 +1318,23 @@ int search_screen(unsigned srchFlags, int mFlags){
   const char *zPattern;
   int fDebug = PB("debug");
   int haveResult = 0;
-  srchFlags = search_restrict(srchFlags);
-  switch( srchFlags ){
+  int srchThisTime;
+  const char *zY = PD("y","all");
+  if( zY[0]=='h' && zY[1]==0 ){
+    srchAllowed = search_restrict(srchAllowed) | (srchAllowed & SRCH_HELP);
+  }else{
+    srchAllowed = search_restrict(srchAllowed);
+  }
+  switch( srchAllowed ){
     case SRCH_CKIN:     zType = " Check-ins";  zClass = "Ckin"; break;
     case SRCH_DOC:      zType = " Docs";       zClass = "Doc";  break;
     case SRCH_TKT:      zType = " Tickets";    zClass = "Tkt";  break;
     case SRCH_WIKI:     zType = " Wiki";       zClass = "Wiki"; break;
     case SRCH_TECHNOTE: zType = " Tech Notes"; zClass = "Note"; break;
     case SRCH_FORUM:    zType = " Forum";      zClass = "Frm";  break;
+    case SRCH_HELP:     zType = " Help";       zClass = "Hlp";  break;
   }
-  if( srchFlags==0 ){
+  if( srchAllowed==0 ){
     if( mFlags & 0x02 ) return 0;
     zDisable1 = " disabled";
     zDisable2 = " disabled";
@@ -1234,8 +1351,13 @@ int search_screen(unsigned srchFlags, int mFlags){
     @ <div class='searchForm'>
   }
   @ <input type="text" name="s" size="40" value="%h(zPattern)"%s(zDisable1)>
-  if( (mFlags & 0x01)!=0 && (srchFlags & (srchFlags-1))!=0 ){
-    static const struct { const char *z; const char *zNm; unsigned m; } aY[] = {
+  srchThisTime = srchAllowed;
+  if( (mFlags & 0x01)!=0 && (srchAllowed & (srchAllowed-1))!=0 ){
+    static const struct {
+      const char *z;
+      const char *zNm;
+      unsigned m;
+    } aY[] = {
        { "all",  "All",        SRCH_ALL      },
        { "c",    "Check-ins",  SRCH_CKIN     },
        { "d",    "Docs",       SRCH_DOC      },
@@ -1243,28 +1365,28 @@ int search_screen(unsigned srchFlags, int mFlags){
        { "w",    "Wiki",       SRCH_WIKI     },
        { "e",    "Tech Notes", SRCH_TECHNOTE },
        { "f",    "Forum",      SRCH_FORUM    },
+       { "h",    "Help",       SRCH_HELP     },
     };
-    const char *zY = PD("y","all");
-    unsigned newFlags = srchFlags;
     int i;
     @ <select size='1' name='y'>
     for(i=0; i<count(aY); i++){
-      if( (aY[i].m & srchFlags)==0 ) continue;
+      if( (aY[i].m & srchAllowed)==0 ) continue;
+      if( aY[i].m==SRCH_HELP && fossil_strcmp(zY,"h")!=0
+       && search_restrict(SRCH_HELP)==0 ) continue;
       cgi_printf("<option value='%s'", aY[i].z);
       if( fossil_strcmp(zY,aY[i].z)==0 ){
-        newFlags &= aY[i].m;
+        srchThisTime &= aY[i].m;
         cgi_printf(" selected");
       }
       cgi_printf(">%s</option>\n", aY[i].zNm);
     }
     @ </select>
-    srchFlags = newFlags;
   }
   if( fDebug ){
     @ <input type="hidden" name="debug" value="1">
   }
   @ <input type="submit" value="Search%s(zType)"%s(zDisable2)>
-  if( srchFlags==0 ){
+  if( srchAllowed==0 && srchThisTime==0 ){
     @ <p class="generalError">Search is disabled</p>
   }
   @ </div></form>
@@ -1275,7 +1397,7 @@ int search_screen(unsigned srchFlags, int mFlags){
     }else{
       @ <div class='searchResult'>
     }
-    if( search_run_and_output(zPattern, srchFlags, fDebug)==0 ){
+    if( search_run_and_output(zPattern, srchThisTime, fDebug)==0 ){
       @ <p class='searchEmpty'>No matches for: <span>%h(zPattern)</span></p>
     }
     @ </div>
@@ -1292,13 +1414,14 @@ int search_screen(unsigned srchFlags, int mFlags){
 **
 **    s=PATTERN       Specify the full-text pattern to search for
 **    y=TYPE          What to search.
-**                      c -> check-ins
-**                      d -> documentation
-**                      t -> tickets
-**                      w -> wiki
-**                      e -> tech notes
-**                      f -> forum
-**                    all -> everything
+**                      c -> check-ins,
+**                      d -> documentation,
+**                      t -> tickets,
+**                      w -> wiki,
+**                      e -> tech notes,
+**                      f -> forum,
+**                      h -> built-in help,
+**                    all -> everything.
 */
 void search_page(void){
   const int isSearch = P("s")!=0;
@@ -1351,7 +1474,7 @@ static void get_stext_by_mimetype(
         wiki_convert(pIn, &html, 0);
       }
     }
-    html_to_plaintext(blob_str(&html), pOut);
+    html_to_plaintext(blob_str(&html), pOut, 0);
   }else if( fossil_strcmp(zMimetype,"text/x-markdown")==0 ){
     markdown_to_html(pIn, blob_size(&title) ? NULL : &title, &html);
   }else if( fossil_strcmp(zMimetype,"text/html")==0 ){
@@ -1360,7 +1483,7 @@ static void get_stext_by_mimetype(
   }
   blob_appendf(pOut, "%s\n", blob_str(&title));
   if( blob_size(pHtml) ){
-    html_to_plaintext(blob_str(pHtml), pOut);
+    html_to_plaintext(blob_str(pHtml), pOut, 0);
   }else{
     blob_append(pOut, blob_buffer(pIn), blob_size(pIn));
   }
@@ -1638,6 +1761,7 @@ static const char zFtsDrop[] =
 @ DROP TABLE IF EXISTS repository.ftsidx;
 @ DROP VIEW IF EXISTS repository.ftscontent;
 @ DROP TABLE IF EXISTS repository.ftsdocs;
+@ DROP TABLE IF EXISTS repository.chatfts1;
 ;
 
 #if INTERFACE
@@ -1683,6 +1807,21 @@ int search_tokenizer_type(int bRecheck){
   return iFtsTokenizer;
 }
 
+/*
+** Returns a string in the form ",tokenize=X", where X is the string
+** counterpart of the given FTS5TOK_xyz value.  Returns "" if tokType
+** does not correspond to a known FTS5 tokenizer.
+*/
+const char * search_tokenize_arg_for_type(int tokType){
+  switch( tokType ){
+    case FTS5TOK_PORTER: return ",tokenize=porter";
+    case FTS5TOK_UNICODE61: return ",tokenize=unicode61";
+    case FTS5TOK_TRIGRAM: return ",tokenize=trigram";
+    case FTS5TOK_NONE:
+    default: return "";
+  }
+}
+
 /*
 ** Returns a string value suitable for use as the search-tokenizer
 ** setting's value, depending on the value of z. If z is 0 then the
@@ -1727,14 +1866,9 @@ void search_set_tokenizer(const char *zName){
 */
 static int searchIdxExists = -1;
 void search_create_index(void){
-  const int useTokenizer = search_tokenizer_type(0);
-  const char *zExtra;
-  switch(useTokenizer){
-    case FTS5TOK_PORTER: zExtra = ",tokenize=porter"; break;
-    case FTS5TOK_UNICODE61: zExtra = ",tokenize=unicode61"; break;
-    case FTS5TOK_TRIGRAM: zExtra = ",tokenize=trigram"; break;
-    default: zExtra = ""; break;
-  }
+  const char *zExtra =
+    search_tokenize_arg_for_type(search_tokenizer_type(0));
+  assert( zExtra );
   search_sql_setup(g.db);
   db_multi_exec(zFtsSchema/*works-like:"%s"*/, zExtra/*safe-for-%s*/);
   searchIdxExists = 1;
@@ -1854,55 +1988,64 @@ void search_doc_touch(char cType, int rid, const char *zName){
 ** changed.
 */
 static void search_update_doc_index(void){
-  const char *zDocBr = db_get("doc-branch","trunk");
-  int ckid = zDocBr ? symbolic_name_to_rid(zDocBr,"ci") : 0;
-  double rTime;
-  if( ckid==0 ) return;
-  if( !db_exists("SELECT 1 FROM ftsdocs WHERE type='c' AND rid=%d"
-                 "   AND NOT idxed", ckid) ) return;
-
-  /* If we get this far, it means that changes to 'd' entries are
-  ** required. */
-  rTime = db_double(0.0, "SELECT mtime FROM event WHERE objid=%d", ckid);
+  const char *zDocBranches = db_get("doc-branch","trunk");
+  int i;
+  Glob * pGlob = glob_create(zDocBranches)
+    /* We're misusing a Glob as a list of comma-/space-delimited
+    ** tokens. We're not actually doing glob matches here. */;
+  if( !pGlob ) return;
   db_multi_exec(
     "CREATE TEMP TABLE current_docs(rid INTEGER PRIMARY KEY, name);"
     "CREATE VIRTUAL TABLE IF NOT EXISTS temp.foci USING files_of_checkin;"
-    "INSERT OR IGNORE INTO current_docs(rid, name)"
-    "  SELECT blob.rid, foci.filename FROM foci, blob"
-    "   WHERE foci.checkinID=%d AND blob.uuid=foci.uuid"
-    "     AND %z",
-    ckid, glob_expr("foci.filename", db_get("doc-glob",""))
-  );
-  db_multi_exec(
-    "DELETE FROM ftsidx WHERE rowid IN"
-    "  (SELECT rowid FROM ftsdocs WHERE type='d'"
-    "      AND rid NOT IN (SELECT rid FROM current_docs))"
-  );
-  db_multi_exec(
-    "DELETE FROM ftsdocs WHERE type='d'"
-    "      AND rid NOT IN (SELECT rid FROM current_docs)"
-  );
-  db_multi_exec(
-    "INSERT OR IGNORE INTO ftsdocs(type,rid,name,idxed,label,bx,url,mtime)"
-    "  SELECT 'd', rid, name, 0,"
-    "         title('d',rid,name),"
-    "         body('d',rid,name),"
-    "         printf('/doc/%T/%%s',urlencode(name)),"
-    "         %.17g"
-    " FROM current_docs",
-    zDocBr, rTime
-  );
-  db_multi_exec(
-    "INSERT INTO ftsidx(rowid,title,body)"
-    "  SELECT rowid, label, bx FROM ftsdocs WHERE type='d' AND NOT idxed"
-  );
-  db_multi_exec(
-    "UPDATE ftsdocs SET"
-    "  idxed=1,"
-    "  bx=NULL,"
-    "  label='Document: '||label"
-    " WHERE type='d' AND NOT idxed"
   );
+  for( i = 0; i < pGlob->nPattern; ++i ){
+    const char *zDocBr = pGlob->azPattern[i];
+    int ckid = symbolic_name_to_rid(zDocBr,"ci");
+    double rTime;
+    if( !db_exists("SELECT 1 FROM ftsdocs WHERE type='c' AND rid=%d"
+                   "   AND NOT idxed", ckid) ) continue;
+    /* If we get this far, it means that changes to 'd' entries are
+    ** required. */
+    rTime = db_double(0.0, "SELECT mtime FROM event WHERE objid=%d", ckid);
+    db_multi_exec(
+      "INSERT OR IGNORE INTO current_docs(rid, name)"
+      "  SELECT blob.rid, foci.filename FROM foci, blob"
+      "   WHERE foci.checkinID=%d AND blob.uuid=foci.uuid"
+      "     AND %z",
+      ckid, glob_expr("foci.filename", db_get("doc-glob",""))
+    );
+    db_multi_exec(
+      "DELETE FROM ftsidx WHERE rowid IN"
+      "  (SELECT rowid FROM ftsdocs WHERE type='d'"
+      "      AND rid NOT IN (SELECT rid FROM current_docs))"
+    );
+    db_multi_exec(
+      "DELETE FROM ftsdocs WHERE type='d'"
+      "      AND rid NOT IN (SELECT rid FROM current_docs)"
+    );
+    db_multi_exec(
+      "INSERT OR IGNORE INTO ftsdocs(type,rid,name,idxed,label,bx,url,mtime)"
+      "  SELECT 'd', rid, name, 0,"
+      "         title('d',rid,name),"
+      "         body('d',rid,name),"
+      "         printf('/doc/%T/%%s',urlencode(name)),"
+      "         %.17g"
+      " FROM current_docs",
+      zDocBr, rTime
+    );
+    db_multi_exec(
+      "INSERT INTO ftsidx(rowid,title,body)"
+      "  SELECT rowid, label, bx FROM ftsdocs WHERE type='d' AND NOT idxed"
+    );
+    db_multi_exec(
+      "UPDATE ftsdocs SET"
+      "  idxed=1,"
+      "  bx=NULL,"
+      "  label='Document: '||label"
+      " WHERE type='d' AND NOT idxed"
+    );
+  }
+  glob_free(pGlob);
 }
 
 /*
@@ -2058,32 +2201,35 @@ void search_rebuild_index(void){
   search_create_index();
   search_fill_index();
   search_update_index(search_restrict(SRCH_ALL));
+  if( db_table_exists("repository","chat") ){
+    chat_rebuild_index(1);
+  }
   fossil_print(" done\n");
 }
 
 /*
-** COMMAND: fts-config*
+** COMMAND: fts-config*                    abbrv-subcom
 **
 ** Usage: fossil fts-config ?SUBCOMMAND? ?ARGUMENT?
 **
 ** The "fossil fts-config" command configures the full-text search capabilities
 ** of the repository.  Subcommands:
 **
-**     reindex            Rebuild the search index.  This is a no-op if
-**                        index search is disabled
+**    reindex            Rebuild the search index.  This is a no-op if
+**                       index search is disabled
 **
-**     index (on|off)     Turn the search index on or off
+**    index (on|off)     Turn the search index on or off
 **
-**     enable cdtwef      Enable various kinds of search. c=Check-ins,
-**                        d=Documents, t=Tickets, w=Wiki, e=Tech Notes,
-**                        f=Forum.
+**    enable TYPE ..     Enable search for TYPE.  TYPE is one of:
+**                       check-in, document, ticket, wiki, technote, 
+**                       forum, help, or all
 **
-**     disable cdtwef     Disable various kinds of search
+**    disable TYPE ...   Disable search for TYPE
 **
-**     tokenizer VALUE    Select a tokenizer for indexed search. VALUE
-**                        may be one of (porter, on, off, trigram, unicode61),
-**                        and "on" is equivalent to "porter". Unindexed
-**                        search never uses tokenization or stemming.
+**    tokenizer VALUE    Select a tokenizer for indexed search. VALUE
+**                       may be one of (porter, on, off, trigram, unicode61),
+**                       and "on" is equivalent to "porter". Unindexed
+**                       search never uses tokenization or stemming.
 **
 ** The current search settings are displayed after any changes are applied.
 ** Run this command with no arguments to simply see the settings.
@@ -2104,12 +2250,13 @@ void fts_config_cmd(void){
     const char *zName;
     const char *zSw;
   } aSetng[] = {
-     { "search-ci",       "check-in search:",  "c" },
-     { "search-doc",      "document search:",  "d" },
-     { "search-tkt",      "ticket search:",    "t" },
-     { "search-wiki",     "wiki search:",      "w" },
-     { "search-technote", "tech note search:", "e" },
-     { "search-forum",    "forum search:",     "f" },
+     { "search-ci",       "check-in search:",       "c" },
+     { "search-doc",      "document search:",       "d" },
+     { "search-tkt",      "ticket search:",         "t" },
+     { "search-wiki",     "wiki search:",           "w" },
+     { "search-technote", "technote search:",       "e" },
+     { "search-forum",    "forum search:",          "f" },
+     { "search-help",     "built-in help search:",  "h" },
   };
   char *zSubCmd = 0;
   int i, j, n;
@@ -2146,12 +2293,42 @@ void fts_config_cmd(void){
 
   /* Adjust search settings */
   if( iCmd==3 || iCmd==4 ){
+    int k;
     const char *zCtrl;
-    if( g.argc<4 ) usage(mprintf("%s STRING",zSubCmd));
-    zCtrl = g.argv[3];
-    for(j=0; j<count(aSetng); j++){
-      if( strchr(zCtrl, aSetng[j].zSw[0])!=0 ){
-        db_set_int(aSetng[j].zSetting/*works-like:"x"*/, iCmd-3, 0);
+    for(k=2; k<g.argc; k++){
+      if( k==2 ){
+        if( g.argc<4 ){
+          zCtrl = "all";
+        }else{
+          zCtrl = g.argv[3];
+          k++;
+        }
+      }else{
+        zCtrl = g.argv[k];
+      }
+      if( fossil_strcmp(zCtrl,"all")==0 ){
+        zCtrl = "cdtwefh";
+      }
+      if( strlen(zCtrl)>=4 ){
+        /* If the argument to "enable" or "disable" is a string of at least
+        ** 4 characters which matches part of any aSetng.zName, then use that
+        ** one aSetng value only. */
+        char *zGlob = mprintf("*%s*", zCtrl);
+        for(j=0; j<count(aSetng); j++){
+          if( sqlite3_strglob(zGlob, aSetng[j].zName)==0 ){
+            db_set_int(aSetng[j].zSetting/*works-like:"x"*/, iCmd-3, 0);
+            zCtrl = 0;
+            break;
+          }
+        }
+        fossil_free(zGlob);
+      }
+      if( zCtrl ){
+        for(j=0; j<count(aSetng); j++){
+          if( strchr(zCtrl, aSetng[j].zSw[0])!=0 ){
+            db_set_int(aSetng[j].zSetting/*works-like:"x"*/, iCmd-3, 0);
+          }
+        }
       }
     }
   }else if( iCmd==5 ){
@@ -2178,10 +2355,10 @@ void fts_config_cmd(void){
 
   /* Always show the status before ending */
   for(i=0; i<count(aSetng); i++){
-    fossil_print("%-17s %s\n", aSetng[i].zName,
+    fossil_print("%-21s %s\n", aSetng[i].zName,
        db_get_boolean(aSetng[i].zSetting,0) ? "on" : "off");
   }
-  fossil_print("%-17s %s\n", "tokenizer:",
+  fossil_print("%-21s %s\n", "tokenizer:",
        search_tokenizer_for_string(0));
   if( search_index_exists() ){
     int pgsz = db_int64(0, "PRAGMA repository.page_size;");
@@ -2190,14 +2367,14 @@ void fts_config_cmd(void){
                                " WHERE schema='repository'"
                                " AND name LIKE 'fts%%'")*pgsz;
     char zSize[50];
-    fossil_print("%-17s FTS%d\n", "full-text index:", search_index_type(1));
-    fossil_print("%-17s %d\n", "documents:",
+    fossil_print("%-21s FTS%d\n", "full-text index:", search_index_type(1));
+    fossil_print("%-21s %d\n", "documents:",
        db_int(0, "SELECT count(*) FROM ftsdocs"));
     approxSizeName(sizeof(zSize), zSize, nFts);
-    fossil_print("%-17s %s (%.1f%% of repository)\n", "space used",
+    fossil_print("%-21s %s (%.1f%% of repository)\n", "space used",
        zSize, 100.0*((double)nFts/(double)nTotal));
   }else{
-    fossil_print("%-17s disabled\n", "full-text index:");
+    fossil_print("%-21s disabled\n", "full-text index:");
   }
   db_end_transaction(0);
 }
