@@ -102,6 +102,7 @@ void secaudit0_page(void){
   const char *zSelfCap;      /* Capabilities of self-registered users */
   int hasSelfReg = 0;        /* True if able to self-register */
   const char *zPublicUrl;    /* Canonical access URL */
+  const char *zVulnReport;   /* The vuln-report setting */
   Blob cmd;
   char *z;
   int n, i;
@@ -365,6 +366,18 @@ void secaudit0_page(void){
     @ of this repository.
   }
 
+  zVulnReport = db_get("vuln-report","log");
+  if( fossil_strcmp(zVulnReport,"block")!=0
+   && fossil_strcmp(zVulnReport,"fatal")!=0
+  ){
+    @ <li><p><b>WARNING:</b>
+    @ The <a href="%R/help?cmd=vuln-report">vuln-report setting</a>
+    @ has a value of "%h(zVulnReport)". This disables defenses against
+    @ XSS or SQL-injection vulnerabilities caused by coding errors in
+    @ custom TH1 scripts.  For the best security, change
+    @ the value of the vuln-report setting to "block" or "fatal".
+  }
+
   /* Obsolete:  */
   if( hasAnyCap(zAnonCap, "d") ||
       hasAnyCap(zDevCap,  "d") ||
@@ -545,7 +558,7 @@ void secaudit0_page(void){
   }
   if( db_get_boolean("http_authentication_ok", 0) ){
     @ <li><p><b>Caution:</b>
-    @ This repository trusts that the HTTP_AUTHENITICATION environment
+    @ This repository trusts that the HTTP_AUTHENTICATION environment
     @ variable set up by the webserver contains the name of an
     @ authenticated user.
     @ Fossil's built-in authentication mechanism is bypassed.
@@ -555,13 +568,13 @@ void secaudit0_page(void){
 
   /* Logging should be turned on
   */
-  if( db_get_boolean("access-log",0)==0 ){
+  if( db_get_boolean("access-log",1)==0 ){
     @ <li><p>
     @ The <a href="access_log">User Log</a> is disabled.  The user log
     @ keeps a record of successful and unsuccessful login attempts and is
     @ useful for security monitoring.
   }
-  if( db_get_boolean("admin-log",0)==0 ){
+  if( db_get_boolean("admin-log",1)==0 ){
     @ <li><p>
     @ The <a href="admin_log">Administrative Log</a> is disabled.
     @ The administrative log provides a record of configuration changes
@@ -578,7 +591,7 @@ void secaudit0_page(void){
     @ filesystem is mounted within the jail, so that the load average
     @ can be obtained from the /proc/loadavg file.
   }else {
-    double r = atof(db_get("max-loadavg", 0));
+    double r = atof(db_get("max-loadavg", "0.0"));
     if( r<=0.0 ){
       @ <li><p>
       @ Load average limiting is turned off.  This can cause the server
@@ -695,6 +708,33 @@ void secaudit0_page(void){
     @ </li>
   }
 
+  @ <li><p>Robot Defenses:
+  @ <ol type="a">
+  switch( db_get_int("auto-hyperlink",1) ){
+    default:
+       @ <li> No auto-enable of hyperlinks.
+       break;
+    case 1:
+       @ <li> Hyperlinks auto-enabled based on UserAgent and Javascript.
+       break;
+    case 2:
+       @ <li> Hyperlinks auto-enabled based on UserAgent only.
+       break;
+  }
+  z = db_get("max-loadavg",0);
+  if( z && atof(z)>0.0 ){
+    @ <li> Maximum load average for expensive requests: %h(z);
+  }else{
+    @ <li> No limits on the load average
+  }
+  z = db_get("robot-restrict",0);
+  if( z==0 ){
+    @ <li> No complex-request constraints on robots
+  }else{
+    @ <li> Complex requests limited for pages matching: %h(z)
+  }
+  @ </ol>
+
   blob_init(&cmd, 0, 0);
   for(i=0; g.argvOrig[i]!=0; i++){
     blob_append_escaped_arg(&cmd, g.argvOrig[i], 0);
@@ -779,32 +819,61 @@ static void no_error_log_available(void){
   }
 }
 
-/*
-** The maximum number of bytes of the error log to show by default.
-*/
-#define MXSHOWLOG 500000
-
 /*
 ** WEBPAGE: errorlog
 **
 ** Show the content of the error log.  Only the administrator can view
 ** this page.
+**
+**    y=0x001          Show only hack attempts
+**    y=0x002          Show only panics and assertion faults
+**    y=0x004          Show hung backoffice processes
+**    y=0x008          Show POST requests from a different origin
+**    y=0x010          Show SQLITE_AUTH and similar
+**    y=0x020          Show SMTP error reports
+**    y=0x040          Show TH1 vulnerability reports
+**    y=0x800          Show other uncategorized messages
+**
+** If y is omitted or is zero, a count of the various message types is
+** shown.
 */
 void errorlog_page(void){
   i64 szFile;
   FILE *in;
   char *zLog;
+  const char *zType = P("y");
+  static const int eAllTypes = 0x87f;
+  long eType = 0;
+  int bOutput = 0;
+  int prevWasTime = 0;
+  int nHack = 0;
+  int nPanic = 0;
+  int nOther = 0;
+  int nHang = 0;
+  int nXPost = 0;
+  int nAuth = 0;
+  int nSmtp = 0;
+  int nVuln = 0;
   char z[10000];
+  char zTime[10000];
+
   login_check_credentials();
   if( !g.perm.Admin ){
     login_needed(0);
     return;
   }
+  if( zType ){
+    eType = strtol(zType,0,0) & eAllTypes;
+  }
   style_header("Server Error Log");
   style_submenu_element("Test", "%R/test-warning");
   style_submenu_element("Refresh", "%R/errorlog");
+  style_submenu_element("Download", "%R/errorlog?download");
+  style_submenu_element("Truncate", "%R/errorlog?truncate");
   style_submenu_element("Log-Menu", "%R/setup-logmenu");
-  style_submenu_element("Panics", "%R/paniclog");
+  if( eType ){
+    style_submenu_element("Summary", "%R/errorlog");
+  }
 
   if( g.zErrlog==0 || fossil_strcmp(g.zErrlog,"-")==0 ){
     no_error_log_available();
@@ -835,79 +904,86 @@ void errorlog_page(void){
   zLog = file_canonical_name_dup(g.zErrlog);
   @ <p>The server error log at "%h(zLog)" is %,lld(szFile) bytes in size.
   fossil_free(zLog);
-  style_submenu_element("Download", "%R/errorlog?download");
-  style_submenu_element("Truncate", "%R/errorlog?truncate");
   in = fossil_fopen(g.zErrlog, "rb");
   if( in==0 ){
     @ <p class='generalError'>Unable to open that file for reading!</p>
     style_finish_page();
     return;
   }
-  if( szFile>MXSHOWLOG && P("all")==0 ){
-    @ <form action="%R/errorlog" method="POST">
-    @ <p>Only the last %,d(MXSHOWLOG) bytes are shown.
-    @ <input type="submit" name="all" value="Show All">
-    @ </form>
-    fseek(in, -MXSHOWLOG, SEEK_END);
+  if( eType==0 ){
+    /* will do a summary */
+  }else if( (eType&eAllTypes)!=eAllTypes ){
+    @ Only the following types of messages displayed:
+    @ <ul>
+    if( eType & 0x01 ){
+      @ <li>Hack attempts
+    }
+    if( eType & 0x02 ){
+      @ <li>Panics and assertion faults
+    }
+    if( eType & 0x04 ){
+      @ <li>Hung backoffice processes
+    }
+    if( eType & 0x08 ){
+      @ <li>POST requests from different origin
+    }
+    if( eType & 0x10 ){
+      @ <li>SQLITE_AUTH and similar errors
+    }
+    if( eType & 0x20 ){
+      @ <li>SMTP malfunctions
+    }
+    if( eType & 0x40 ){
+      @ <li>TH1 vulnerabilities
+    }
+    if( eType & 0x800 ){
+      @ <li>Other uncategorized messages
+    }
+    @ </ul>
   }
   @ <hr>
-  @ <pre>
+  if( eType ){
+    @ <pre>
+  }
   while( fgets(z, sizeof(z), in) ){
-    @ %h(z)\
-  }
-  fclose(in);
-  @ </pre>
-  style_finish_page();
-}
-
-/*
-** WEBPAGE: paniclog
-**
-** Scan the error log for panics.  Show all panic messages, ignoring all
-** other error log entries.
-*/
-void paniclog_page(void){
-  i64 szFile;
-  char *zLog;
-  FILE *in;
-  int bOutput = 0;
-  int prevWasTime = 0;
-  char z[10000];
-  char zTime[10000];
-
-  login_check_credentials();
-  if( !g.perm.Admin ){
-    login_needed(0);
-    return;
-  }
-  style_header("Server Panic Log");
-  style_submenu_element("Log-Menu", "%R/setup-logmenu");
-
-  if( g.zErrlog==0 || fossil_strcmp(g.zErrlog,"-")==0 ){
-    no_error_log_available();
-    style_finish_page();
-    return;
-  }
-  in = fossil_fopen(g.zErrlog, "rb");
-  if( in==0 ){
-    @ <p class='generalError'>Unable to open that file for reading!</p>
-    style_finish_page();
-    return;
-  }
-  szFile = file_size(g.zErrlog, ExtFILE);
-  zLog = file_canonical_name_dup(g.zErrlog);
-  @ Panic messages contained within the %lld(szFile)-byte 
-  @ <a href="%R/errorlog?all">error log</a> found at
-  @ "%h(zLog)".
-  fossil_free(zLog);
-  @ <hr>
-  @ <pre>
-  while( fgets(z, sizeof(z), in) ){
-    if( prevWasTime
-     && (strncmp(z,"panic: ", 7)==0 || strstr(z," assertion fault ")!=0)
-    ){
-      @ %h(zTime)\
-      bOutput = 1;
+    if( prevWasTime ){
+      if( strncmp(z,"possible hack attempt - 418 ", 27)==0 ){
+        bOutput = (eType & 0x01)!=0;
+        nHack++;
+      }else
+      if( (strncmp(z,"panic: ", 7)==0 || strstr(z," assertion fault ")!=0) ){
+        bOutput = (eType & 0x02)!=0;
+        nPanic++;
+      }else
+      if( strncmp(z,"SMTP:", 5)==0 ){
+        bOutput = (eType & 0x20)!=0;
+        nSmtp++;
+      }else
+      if( sqlite3_strglob("warning: backoffice process * still *",z)==0 ){
+        bOutput = (eType & 0x04)!=0;
+        nHang++;
+      }else
+      if( sqlite3_strglob("warning: POST from different origin*",z)==0 ){
+        bOutput = (eType & 0x08)!=0;
+        nXPost++;
+      }else
+      if( sqlite3_strglob("SECURITY: authorizer blocks*",z)==0
+       || sqlite3_strglob("warning: SQLITE_AUTH*",z)==0
+      ){
+        bOutput = (eType & 0x10)!=0;
+        nAuth++;
+      }else
+      if( strncmp(z,"possible", 8)==0 && strstr(z,"tainted")!=0 ){
+        bOutput = (eType & 0x40)!=0;
+        nVuln++;
+      }else
+      {
+        bOutput = (eType & 0x800)!=0;
+        nOther++;
+      }
+      if( bOutput ){
+        @ %h(zTime)\
+      }
     }
     if( strncmp(z, "--------", 8)==0 ){
       size_t n = strlen(z);
@@ -917,11 +993,57 @@ void paniclog_page(void){
     }else{
       prevWasTime = 0;
     }
-    if( bOutput ){
+    if( bOutput && eType ){
       @ %h(z)\
     }
   }
   fclose(in);
-  @ </pre>
+  if( eType ){
+    @ </pre>
+  }
+  if( eType==0 ){
+    int nNonHack = nPanic + nHang + nAuth + nSmtp + nVuln + nOther;
+    int nTotal = nNonHack + nHack + nXPost;
+    @ <p><table border="a" cellspacing="0" cellpadding="5">
+    if( nPanic>0 ){
+      @ <tr><td align="right">%d(nPanic)</td>
+      @     <td><a href="./errorlog?y=2">Panics</a></td>
+    }
+    if( nVuln>0 ){
+      @ <tr><td align="right">%d(nVuln)</td>
+      @     <td><a href="./errorlog?y=64">TH1 Vulnerabilities</a></td>
+    }
+    if( nHack>0 ){
+      @ <tr><td align="right">%d(nHack)</td>
+      @     <td><a href="./errorlog?y=1">Hack Attempts</a></td>
+    }
+    if( nHang>0 ){
+      @ <tr><td align="right">%d(nHang)</td>
+      @     <td><a href="./errorlog?y=4">Hung Backoffice</a></td>
+    }
+    if( nXPost>0 ){
+      @ <tr><td align="right">%d(nXPost)</td>
+      @     <td><a href="./errorlog?y=8">POSTs from different origin</a></td>
+    }
+    if( nAuth>0 ){
+      @ <tr><td align="right">%d(nAuth)</td>
+      @     <td><a href="./errorlog?y=16">SQLITE_AUTH and similar</a></td>
+    }
+    if( nSmtp>0 ){
+      @ <tr><td align="right">%d(nSmtp)</td>
+      @     <td><a href="./errorlog?y=32">SMTP faults</a></td>
+    }
+    if( nOther>0 ){
+      @ <tr><td align="right">%d(nOther)</td>
+      @     <td><a href="./errorlog?y=2048">Other</a></td>
+    }
+    @ <tr><td align="right">%d(nTotal)</td>
+    if( nTotal>0 ){
+      @     <td><a href="./errorlog?y=4095">All Messages</a></td>
+    }else{
+      @     <td>All Messages</td>
+    }
+    @ </table>
+  }
   style_finish_page();
 }
