@@ -43,17 +43,18 @@ void setup_ulist(void){
   const char *zWith = P("with");
   int bUnusedOnly = P("unused")!=0;
   int bUbg = P("ubg")!=0;
+  int bHaveAlerts;
 
   login_check_credentials();
   if( !g.perm.Admin ){
     login_needed(0);
     return;
   }
-
+  bHaveAlerts = alert_tables_exist();
   style_submenu_element("Add", "setup_uedit");
   style_submenu_element("Log", "access_log");
   style_submenu_element("Help", "setup_ulist_notes");
-  if( alert_tables_exist() ){
+  if( bHaveAlerts ){
     style_submenu_element("Subscribers", "subscribers");
   }
   style_set_current_feature("setup");
@@ -117,7 +118,7 @@ void setup_ulist(void){
     style_submenu_element("Unused", "setup_ulist?unused");
   }
   @ <table border=1 cellpadding=2 cellspacing=0 class='userTable sortable' \
-  @  data-column-types='ktxTTKt' data-init-sort='2'>
+  @  data-column-types='ktxKTKt' data-init-sort='4'>
   @ <thead><tr>
   @ <th>Login Name<th>Caps<th>Info<th>Date<th>Expire<th>Last Login\
   @ <th>Alerts</tr></thead>
@@ -149,7 +150,7 @@ void setup_ulist(void){
         "SELECT user FROM event WHERE user NOT NULL "
         "UNION ALL SELECT euser FROM event WHERE euser NOT NULL%s)"
         " AND uid NOT IN (SELECT uid FROM rcvfrom)",
-        alert_tables_exist() ?
+        bHaveAlerts ?
           " UNION ALL SELECT suname FROM subscriber WHERE suname NOT NULL":"");
   }else if( zWith && zWith[0] ){
     zWith = mprintf(" AND fullcap(cap) GLOB '*[%q]*'", zWith);
@@ -157,17 +158,22 @@ void setup_ulist(void){
     zWith = "";
   }
   db_prepare(&s,
-     "SELECT uid, login, cap, info, date(user.mtime,'unixepoch'),"
-     "       lower(login) AS sortkey, "
-     "       CASE WHEN info LIKE '%%expires 20%%'"
+      /*0-4*/"SELECT uid, login, cap, info, date(user.mtime,'unixepoch'),"
+      /* 5 */"lower(login) AS sortkey, "
+      /* 6 */"CASE WHEN info LIKE '%%expires 20%%'"
              "    THEN substr(info,instr(lower(info),'expires')+8,10)"
              "    END AS exp,"
-             "atime,"
-     "       subscriber.ssub, subscriber.subscriberId"
-     "  FROM user LEFT JOIN lastAccess ON login=uname"
-     "            LEFT JOIN subscriber ON login=suname"
-     " WHERE login NOT IN ('anonymous','nobody','developer','reader') %s"
-     " ORDER BY sortkey", zWith/*safe-for-%s*/
+      /* 7 */"atime,"
+      /* 8 */"user.mtime AS sorttime,"
+      /*9-11*/"%s"
+             " FROM user LEFT JOIN lastAccess ON login=uname"
+             "            LEFT JOIN subscriber ON login=suname"
+             " WHERE login NOT IN ('anonymous','nobody','developer','reader') %s"
+             " ORDER BY sorttime DESC",
+             bHaveAlerts
+             ? "subscriber.ssub, subscriber.subscriberId, subscriber.semail"
+             : "null, null, null",
+             zWith/*safe-for-%s*/
   );
   rNow = db_double(0.0, "SELECT julianday('now');");
   while( db_step(&s)==SQLITE_ROW ){
@@ -181,7 +187,8 @@ void setup_ulist(void){
     double rATime = db_column_double(&s,7);
     char *zAge = 0;
     const char *zSub;
-    int sid = db_column_int(&s,9);
+    int sid = db_column_int(&s,10);
+    sqlite3_int64 sorttime = db_column_int64(&s, 8);
     if( rATime>0.0 ){
       zAge = human_readable_age(rNow - rATime);
     }
@@ -194,15 +201,17 @@ void setup_ulist(void){
     @ <a href='setup_uedit?id=%d(uid)'>%h(zLogin)</a>
     @ <td>%h(zCap)
     @ <td>%h(zInfo)
-    @ <td>%h(zDate?zDate:"")
+    @ <td data-sortkey='%09llx(sorttime)'>%h(zDate?zDate:"")
     @ <td>%h(zExp?zExp:"")
     @ <td data-sortkey='%f(rATime)' style='white-space:nowrap'>%s(zAge?zAge:"")
-    if( db_column_type(&s,8)==SQLITE_NULL ){
+    if( db_column_type(&s,9)==SQLITE_NULL ){
       @ <td>
-    }else if( (zSub = db_column_text(&s,8))==0 || zSub[0]==0 ){
+    }else if( (zSub = db_column_text(&s,9))==0 || zSub[0]==0 ){
       @ <td><a href="%R/alerts?sid=%d(sid)"><i>off</i></a>
     }else{
-      @ <td><a href="%R/alerts?sid=%d(sid)">%h(zSub)</a>
+      const char *zEmail = db_column_text(&s, 11);
+      char * zAt = zEmail ? mprintf(" &rarr; %h", zEmail) : mprintf("");
+      @ <td><a href="%R/alerts?sid=%d(sid)">%h(zSub)</a>  %z(zAt)
     }
 
     @ </tr>
@@ -303,6 +312,134 @@ static int isValidPwString(const char *zPw){
   return zPw[0]!=0;
 }
 
+/*
+** Return true if user capability strings zOrig and zNew materially
+** differ, taking into account that they may be sorted in an arbitary
+** order. This does not take inherited permissions into
+** account. Either argument may be NULL. A NULL and an empty string
+** are considered equivalent here. e.g. "abc" and "cab" are equivalent
+** for this purpose, but "aCb" and "acb" are not.
+*/
+static int userCapsChanged(const char *zOrig, const char *zNew){
+  if( !zOrig ){
+    return zNew ? (0!=*zNew) : 0;
+  }else if( !zNew ){
+    return 0!=*zOrig;
+  }else if( 0==fossil_strcmp(zOrig, zNew) ){
+    return 0;
+  }else{
+    /* We don't know that zOrig and zNew are sorted equivalently.  The
+    ** following steps will compare strings which contain all the same
+    ** capabilities letters as equivalent, regardless of the letters'
+    ** order in their strings. */
+    char aOrig[128]; /* table of zOrig bytes */
+    int nOrig = 0, nNew = 0;
+
+    memset( &aOrig[0], 0, sizeof(aOrig) );
+    for( ; *zOrig; ++zOrig, ++nOrig ){
+      if( 0==(*zOrig & 0x80) ){
+        aOrig[(int)*zOrig] = 1;
+      }
+    }
+    for( ; *zNew; ++zNew, ++nNew ){
+      if( 0==(*zNew & 0x80) && !aOrig[(int)*zNew] ){
+        return 1;
+      }
+    }
+    return nOrig!=nNew;
+  }
+}
+
+/*
+** COMMAND: test-user-caps-changed
+**
+** Usage: %fossil test-user-caps-changed caps1 caps2
+**
+*/
+void test_user_caps_changed(void){
+
+  char const * zOld = g.argc>2 ? g.argv[2] : NULL;
+  char const * zNew = g.argc>3 ? g.argv[3] : NULL;
+  fossil_print("Has changes? = %d\n",
+               userCapsChanged( zOld, zNew ));
+}
+
+/*
+** Sends notification of user permission elevation changes to all
+** subscribers with a "u" subscription. This is a no-op if alerts are
+** not enabled.
+**
+** These subscriptions differ from most, in that:
+**
+** - They currently lack an "unsubscribe" link.
+**
+** - Only an admin can assign this subscription, but if a non-admin
+**   edits their subscriptions after an admin assigns them this one,
+**   this particular one will be lost.  "Feature or bug?" is unclear,
+**   but it would be odd for a non-admin to be assigned this
+**   capability.
+*/
+static void alert_user_cap_change(const char *zLogin,   /*Affected user*/
+                                 int uid,              /*[user].uid*/
+                                 int bIsNew,           /*true if new user*/
+                                 const char *zOrigCaps,/*Old caps*/
+                                 const char *zNewCaps  /*New caps*/){
+  Blob hdr, body;
+  Stmt q;
+  int nBody;
+  AlertSender *pSender;
+  char *zSubname;
+  char *zURL;
+  char * zSubject;
+
+  if( !alert_enabled() ) return;
+  zSubject = bIsNew
+    ? mprintf("New user created: [%q]", zLogin)
+    : mprintf("User [%q] capabilities changed", zLogin);
+  zURL = db_get("email-url",0);
+  zSubname = db_get("email-subname", "[Fossil Repo]");
+  blob_init(&body, 0, 0);
+  blob_init(&hdr, 0, 0);
+  if( bIsNew ){
+    blob_appendf(&body, "User [%q] was created with "
+                 "permissions [%q] by user [%q].\n",
+                 zLogin, zNewCaps, g.zLogin);
+  } else {
+    blob_appendf(&body, "Permissions for user [%q] where changed "
+                 "from [%q] to [%q] by user [%q].\n",
+                 zLogin, zOrigCaps, zNewCaps, g.zLogin);
+  }
+  if( zURL ){
+    blob_appendf(&body, "\nUser editor: %s/setup_uedit?uid=%d\n", zURL, uid);
+  }
+  nBody = blob_size(&body);
+  pSender = alert_sender_new(0, 0);
+  db_prepare(&q,
+        "SELECT semail, hex(subscriberCode)"
+        "  FROM subscriber, user "
+        " WHERE sverified AND NOT sdonotcall"
+        "   AND suname=login"
+        "   AND ssub GLOB '*u*'");
+  while( !pSender->zErr && db_step(&q)==SQLITE_ROW ){
+    const char *zTo = db_column_text(&q, 0);
+    blob_truncate(&hdr, 0);
+    blob_appendf(&hdr, "To: <%s>\r\nSubject: %s %s\r\n",
+                 zTo, zSubname, zSubject);
+    if( zURL ){
+      const char *zCode = db_column_text(&q, 1);
+      blob_truncate(&body, nBody);
+      blob_appendf(&body,"\n-- \nSubscription info: %s/alerts/%s\n",
+                   zURL, zCode);
+    }
+    alert_send(pSender, &hdr, &body, 0);
+  }
+  db_finalize(&q);
+  alert_sender_free(pSender);
+  fossil_free(zURL);
+  fossil_free(zSubname);
+  fossil_free(zSubject);
+}
+
 /*
 ** WEBPAGE: setup_uedit
 **
@@ -314,6 +451,7 @@ void user_edit(void){
   const char *zGroup;
   const char *zOldLogin;
   int uid, i;
+  char *zOldCaps = 0;        /* Capabilities before edit */
   char *zDeleteVerify = 0;   /* Delete user verification text */
   int higherUser = 0;  /* True if user being edited is SETUP and the */
                        /* user doing the editing is ADMIN.  Disallow editing */
@@ -331,10 +469,11 @@ void user_edit(void){
   */
   zId = PD("id", "0");
   uid = atoi(zId);
-  if( zId && !g.perm.Setup && uid>0 ){
-    char *zOldCaps;
-    zOldCaps = db_text(0, "SELECT cap FROM user WHERE uid=%d",uid);
-    higherUser = zOldCaps && strchr(zOldCaps,'s');
+  if( uid>0 ){
+    zOldCaps = db_text("", "SELECT cap FROM user WHERE uid=%d",uid);
+    if( zId && !g.perm.Setup ){
+      higherUser = zOldCaps && strchr(zOldCaps,'s');
+    }
   }
 
   if( P("can") ){
@@ -393,26 +532,29 @@ void user_edit(void){
   }else{
     /* We have all the information we need to make the change to the user */
     char c;
-    char zCap[70], zNm[4];
+    int bCapsChanged = 0 /* 1 if user's permissions changed */;
+    const int bIsNew = uid<=0;
+    char aCap[70], zNm[4];
     zNm[0] = 'a';
     zNm[2] = 0;
     for(i=0, c='a'; c<='z'; c++){
       zNm[1] = c;
       a[c&0x7f] = ((c!='s' && c!='y') || g.perm.Setup) && P(zNm)!=0;
-      if( a[c&0x7f] ) zCap[i++] = c;
+      if( a[c&0x7f] ) aCap[i++] = c;
     }
     for(c='0'; c<='9'; c++){
       zNm[1] = c;
       a[c&0x7f] = P(zNm)!=0;
-      if( a[c&0x7f] ) zCap[i++] = c;
+      if( a[c&0x7f] ) aCap[i++] = c;
     }
     for(c='A'; c<='Z'; c++){
       zNm[1] = c;
       a[c&0x7f] = P(zNm)!=0;
-      if( a[c&0x7f] ) zCap[i++] = c;
+      if( a[c&0x7f] ) aCap[i++] = c;
     }
 
-    zCap[i] = 0;
+    aCap[i] = 0;
+    bCapsChanged = bIsNew || userCapsChanged(zOldCaps, &aCap[0]);
     zPw = P("pw");
     zLogin = P("login");
     if( strlen(zLogin)==0 ){
@@ -444,11 +586,12 @@ void user_edit(void){
     }
     cgi_csrf_verify();
     db_unprotect(PROTECT_USER);
-    db_multi_exec(
-       "REPLACE INTO user(uid,login,info,pw,cap,mtime) "
-       "VALUES(nullif(%d,0),%Q,%Q,%Q,%Q,now())",
-      uid, zLogin, P("info"), zPw, zCap
-    );
+    uid = db_int(0,
+                 "REPLACE INTO user(uid,login,info,pw,cap,mtime) "
+                 "VALUES(nullif(%d,0),%Q,%Q,%Q,%Q,now()) "
+                 "RETURNING uid",
+                 uid, zLogin, P("info"), zPw, &aCap[0]);
+    assert( uid>0 );
     if( zOldLogin && fossil_strcmp(zLogin, zOldLogin)!=0 ){
       if( alert_tables_exist() ){
         /* Rename matching subscriber entry, else the user cannot
@@ -460,8 +603,9 @@ void user_edit(void){
     }
     db_protect_pop();
     setup_incr_cfgcnt();
-    admin_log( "Updated user [%q] with capabilities [%q].",
-               zLogin, zCap );
+    admin_log( "%s user [%q] with capabilities [%q].",
+               bIsNew ? "Added" : "Updated",
+               zLogin, &aCap[0] );
     if( atoi(PD("all","0"))>0 ){
       Blob sql;
       char *zErr = 0;
@@ -496,7 +640,7 @@ void user_edit(void){
         "  cap=%Q,"
         "  mtime=now()"
         " WHERE login=%Q;",
-        zLogin, P("pw"), zLogin, P("info"), zCap,
+        zLogin, P("pw"), zLogin, P("info"), &aCap[0],
         zOldLogin
       );
       db_unprotect(PROTECT_USER);
@@ -505,7 +649,7 @@ void user_edit(void){
       blob_reset(&sql);
       admin_log( "Updated user [%q] in all login groups "
                  "with capabilities [%q].",
-                 zLogin, zCap );
+                 zLogin, &aCap[0] );
       if( zErr ){
         const char *zRef = cgi_referer("setup_ulist");
         style_header("User Change Error");
@@ -515,9 +659,17 @@ void user_edit(void){
         @ <p><a href="setup_uedit?id=%d(uid)&referer=%T(zRef)">
         @ [Bummer]</a></p>
         style_finish_page();
+        if( bCapsChanged ){
+          /* It's possible that caps were updated locally even if
+          ** login group updates failed. */
+          alert_user_cap_change(zLogin, uid, bIsNew, zOldCaps, &aCap[0]);
+        }
         return;
       }
     }
+    if( bCapsChanged ){
+      alert_user_cap_change(zLogin, uid, bIsNew, zOldCaps, &aCap[0]);
+    }
     cgi_redirect(cgi_referer("setup_ulist"));
     return;
   }
@@ -526,15 +678,15 @@ void user_edit(void){
   */
   zLogin = "";
   zInfo = "";
-  zCap = "";
+  zCap = zOldCaps;
   zPw = "";
   for(i='a'; i<='z'; i++) oa[i] = "";
   for(i='0'; i<='9'; i++) oa[i] = "";
   for(i='A'; i<='Z'; i++) oa[i] = "";
   if( uid ){
+    assert( zCap );
     zLogin = db_text("", "SELECT login FROM user WHERE uid=%d", uid);
     zInfo = db_text("", "SELECT info FROM user WHERE uid=%d", uid);
-    zCap = db_text("", "SELECT cap FROM user WHERE uid=%d", uid);
     zPw = db_text("", "SELECT pw FROM user WHERE uid=%d", uid);
     for(i=0; zCap[i]; i++){
       char c = zCap[i];
@@ -1018,7 +1170,7 @@ void setup_uinfo_page(void){
   @ <tr><th>uid:</th><td>%d(db_column_int(&q,0))
   @  (<a href="%R/setup_uedit?id=%d(db_column_int(&q,0))">edit</a>)</td></tr>
   @ <tr><th>login:</th><td>%h(db_column_text(&q,1))</td></tr>
-  @ <tr><th>capabilities:</th><td>%h(db_column_text(&q,2))</th></tr>
+  @ <tr><th>capabilities:</th><td>%h(db_column_text(&q,2))</td></tr>
   @ <tr><th valign="top">info:</th>
   @ <td valign="top"><span style='white-space:pre-line;'>\
   @ %h(db_column_text(&q,5))</span></td></tr>
