@@ -168,7 +168,7 @@ void stat_page(void){
     style_submenu_element("Table Sizes", "repo-tabsize");
   }
   if( g.perm.Admin || g.perm.Setup || db_get_boolean("test_env_enable",0) ){
-    style_submenu_element("Environment", "test_env");
+    style_submenu_element("Environment", "test-env");
   }
   @ <table class="label-value">
   fsize = file_size(g.zRepositoryName, ExtFILE);
@@ -559,7 +559,7 @@ void urllist_page(void){
     cnt++;
   }
   db_finalize(&q);
-  
+
   if( nOmitted ){
     @ <tr><td><a href="urllist?all"><i>Show %d(nOmitted) more...</i></a>
   }
@@ -717,6 +717,22 @@ void repo_schema_page(void){
   login_check_credentials();
   if( !g.perm.Admin ){ login_needed(0); return; }
 
+  if( zArg!=0
+   && db_table_exists("repository",zArg)
+   && cgi_csrf_safe(1)
+  ){
+    if( P("analyze")!=0 ){
+      db_multi_exec("ANALYZE \"%w\"", zArg);
+    }else if( P("analyze200")!=0 ){
+      db_multi_exec("PRAGMA analysis_limit=200; ANALYZE \"%w\"", zArg);
+    }else if( P("deanalyze")!=0 ){
+      db_unprotect(PROTECT_ALL);
+      db_multi_exec("DELETE FROM repository.sqlite_stat1"
+                    " WHERE tbl LIKE %Q", zArg);
+      db_protect_pop();
+    }
+  }
+
   style_set_current_feature("stat");
   style_header("Repository Schema");
   style_adunit_config(ADUNIT_RIGHT_OK);
@@ -761,6 +777,13 @@ void repo_schema_page(void){
       style_submenu_element("Stat1","repo_stat1");
     }
   }
+  @ <hr><form method="POST">
+  @ <input type="submit" name="analyze" value="Run ANALYZE"><br />
+  @ <input type="submit" name="analyze200"\
+  @  value="Run ANALYZE with limit=200"><br />
+  @ <input type="submit" name="deanalyze" value="De-ANALYZE">
+  @ </form>
+
   style_finish_page();
 }
 
@@ -770,30 +793,66 @@ void repo_schema_page(void){
 ** Show the sqlite_stat1 table for the repository schema
 */
 void repo_stat1_page(void){
+  int bTabular;
   login_check_credentials();
   if( !g.perm.Admin ){ login_needed(0); return; }
+  bTabular = PB("tabular");
 
+  if( P("analyze")!=0 && cgi_csrf_safe(1) ){
+    db_multi_exec("ANALYZE");
+  }else if( P("analyze200")!=0 && cgi_csrf_safe(1) ){
+    db_multi_exec("PRAGMA analysis_limit=200; ANALYZE;");
+  }else if( P("deanalyze")!=0 && cgi_csrf_safe(1) ){
+    db_unprotect(PROTECT_ALL);
+    db_multi_exec("DELETE FROM repository.sqlite_stat1;");
+    db_protect_pop();
+  }
   style_set_current_feature("stat");
   style_header("Repository STAT1 Table");
   style_adunit_config(ADUNIT_RIGHT_OK);
   style_submenu_element("Stat", "stat");
   style_submenu_element("Schema", "repo_schema");
+  style_submenu_checkbox("tabular", "Tabular", 0, 0);
   if( db_table_exists("repository","sqlite_stat1") ){
     Stmt q;
     db_prepare(&q,
       "SELECT tbl, idx, stat FROM repository.sqlite_stat1"
       " ORDER BY tbl, idx");
-    @ <pre>
+    if( bTabular ){
+      @ <table border="1" cellpadding="0" cellspacing="0">
+      @ <tr><th>Table<th>Index<th>Stat
+    }else{
+      @ <pre>
+    }
     while( db_step(&q)==SQLITE_ROW ){
       const char *zTab = db_column_text(&q,0);
       const char *zIdx = db_column_text(&q,1);
       const char *zStat = db_column_text(&q,2);
       char *zUrl = href("%R/repo_schema?n=%t",zTab);
-      @ INSERT INTO sqlite_stat1 VALUES('%z(zUrl)%h(zTab)</a>','%h(zIdx)','%h(zStat)');
+      if( bTabular ){
+        @ <tr><td>%z(zUrl)%h(zTab)</a><td>%h(zIdx)<td>%h(zStat)
+      }else{
+        @ INSERT INTO sqlite_stat1 \
+        @ VALUES('%z(zUrl)%h(zTab)</a>','%h(zIdx)','%h(zStat)');
+      }
+    }
+    if( bTabular ){
+      @ </table>
+    }else{
+      @ </pre>
     }
-    @ </pre>
     db_finalize(&q);
   }
+  @ <p><form method="POST">
+  if( bTabular ){
+    @ <input type="hidden" name="tabular" value="1">
+  }
+  @ <input type="submit" name="analyze" value="Run ANALYZE"><br />
+  @ <input type="submit" name="analyze200"\
+  @  value="Run ANALYZE with limit=200"><br>
+  @ <input type="submit" name="deanalyze"\
+  @  value="De-ANALYZE">
+  @ </form>
   style_finish_page();
 }
 
@@ -877,7 +936,7 @@ void repo_tabsize_page(void){
 ** Only populate the artstat.atype field if the bWithTypes parameter is true.
 */
 void gather_artifact_stats(int bWithTypes){
-  static const char zSql[] = 
+  static const char zSql[] =
     @ CREATE TEMP TABLE artstat(
     @   id INTEGER PRIMARY KEY,   -- Corresponds to BLOB.RID
     @   atype TEXT,               -- 'data', 'manifest', 'tag', 'wiki', etc.
@@ -892,7 +951,7 @@ void gather_artifact_stats(int bWithTypes){
     @      FROM blob LEFT JOIN delta ON blob.rid=delta.rid
     @     WHERE content IS NOT NULL;
   ;
-  static const char zSql2[] = 
+  static const char zSql2[] =
     @ UPDATE artstat SET atype='file'
     @  WHERE +id IN (SELECT fid FROM mlink);
     @ UPDATE artstat SET atype='manifest'
@@ -900,34 +959,34 @@ void gather_artifact_stats(int bWithTypes){
     @ UPDATE artstat SET atype='forum'
     @  WHERE id IN (SELECT objid FROM event WHERE type='f') AND atype IS NULL;
     @ UPDATE artstat SET atype='cluster'
-    @  WHERE atype IS NULL 
+    @  WHERE atype IS NULL
     @    AND id IN (SELECT rid FROM tagxref
     @                WHERE tagid=(SELECT tagid FROM tag
     @                              WHERE tagname='cluster'));
     @ UPDATE artstat SET atype='ticket'
-    @  WHERE atype IS NULL 
+    @  WHERE atype IS NULL
     @    AND id IN (SELECT rid FROM tagxref
     @                WHERE tagid IN (SELECT tagid FROM tag
     @                              WHERE tagname GLOB 'tkt-*'));
     @ UPDATE artstat SET atype='wiki'
-    @  WHERE atype IS NULL 
+    @  WHERE atype IS NULL
     @    AND id IN (SELECT rid FROM tagxref
     @                WHERE tagid IN (SELECT tagid FROM tag
     @                              WHERE tagname GLOB 'wiki-*'));
     @ UPDATE artstat SET atype='technote'
-    @  WHERE atype IS NULL 
+    @  WHERE atype IS NULL
     @    AND id IN (SELECT rid FROM tagxref
     @                WHERE tagid IN (SELECT tagid FROM tag
     @                              WHERE tagname GLOB 'event-*'));
     @ UPDATE artstat SET atype='attachment'
-    @  WHERE atype IS NULL 
-    @    AND id IN (SELECT attachid FROM attachment UNION 
+    @  WHERE atype IS NULL
+    @    AND id IN (SELECT attachid FROM attachment UNION
     @               SELECT blob.rid FROM attachment JOIN blob ON uuid=src);
     @ UPDATE artstat SET atype='tag'
-    @  WHERE atype IS NULL 
+    @  WHERE atype IS NULL
     @    AND id IN (SELECT srcid FROM tagxref);
     @ UPDATE artstat SET atype='tag'
-    @  WHERE atype IS NULL 
+    @  WHERE atype IS NULL
     @    AND id IN (SELECT objid FROM event WHERE type='g');
     @ UPDATE artstat SET atype='unused' WHERE atype IS NULL;
   ;
