@@ -220,8 +220,8 @@ static int contains_whitespace(const char *zName){
 }
 
 /*
-** COMMAND: uv#
-** COMMAND: unversioned
+** COMMAND: uv#                           abbrv-subcom
+** COMMAND: unversioned                   abbrv-subcom
 **
 ** Usage: %fossil unversioned SUBCOMMAND ARGS...
 **    or: %fossil uv SUBCOMMAND ARGS..
@@ -248,6 +248,8 @@ static int contains_whitespace(const char *zName){
 **    cat FILE ...           Concatenate the content of FILEs to stdout.
 **
 **    edit FILE              Bring up FILE in a text editor for modification.
+**                           Options:
+**                             --editor NAME     Name of the text editor to use
 **
 **    export FILE OUTPUT     Write the content of FILE into OUTPUT on disk
 **
@@ -268,6 +270,7 @@ static int contains_whitespace(const char *zName){
 **                           Options:
 **                              -v|--verbose     Extra diagnostic output
 **                              -n|--dry-run     Show what would have happened
+**                              --proxy PROXY    Use the specified HTTP proxy
 **
 **    remove|rm|delete FILE ...
 **                           Remove unversioned files from the local repository.
@@ -287,13 +290,14 @@ static int contains_whitespace(const char *zName){
 **                           Options:
 **                              -v|--verbose     Extra diagnostic output
 **                              -n|--dry-run     Show what would have happened
+**                              --proxy PROXY    Use the specified HTTP proxy
 **
 **    touch FILE ...         Update the TIMESTAMP on all of the listed files
 **
 ** Options:
 **   --mtime TIMESTAMP       Use TIMESTAMP instead of "now" for the "add",
 **                           "edit", "remove", and "touch" subcommands.
-**   -R|--repository REPO    Use FILE as the repository
+**   -R|--repository REPO    Use REPO as the repository
 */
 void unversioned_cmd(void){
   const char *zCmd;
@@ -361,13 +365,13 @@ void unversioned_cmd(void){
     char *zCmd;             /* Command to run the text editor */
     Blob content;           /* Content of the unversioned file */
 
-    verify_all_options();
-    if( g.argc!=4) usage("edit UVFILE");
-    zUVFile = g.argv[3];
     zEditor = fossil_text_editor();
     if( zEditor==0 ){
       fossil_fatal("no text editor - set the VISUAL env variable");
     }
+    verify_all_options();
+    if( g.argc!=4) usage("edit UVFILE");
+    zUVFile = g.argv[3];
     zTFile = fossil_temp_filename();
     if( zTFile==0 ) fossil_fatal("cannot find a temporary filename");
     db_begin_transaction();
@@ -523,6 +527,60 @@ void unversioned_cmd(void){
   }
 }
 
+/*
+** Emit an HTML form for uploading a new unversioned file if
+** the current user has WrUnver permissions, else this is
+** a no-op.
+**
+** If this function detects that the form it emits has been submitted,
+** it will add the uploaded file to the unversioned file list before
+** returning.
+**
+** Intended only for use by /uvlist, and its form's action is that
+** page.
+*/
+static void uvlist_upload(void){
+  const char * aContent;
+  if( !g.perm.WrUnver ) return;
+  aContent = P("f");
+  if( aContent!=0 ){
+    const char * const zName = P("f:filename");
+    int const nContent = atoi(PD("f:bytes","0"));
+    const char * zError = 0;
+    Blob content;
+    if( zName[0]==0 ){
+      zError = "be an empty string";
+    }else if( contains_whitespace(zName) ){
+      zError = "contain spaces";
+    }
+    if( zError ){
+      fossil_fatal("Unversioned filenames may not %s: %h",
+                   zError, zName);
+    }
+    unversioned_schema();
+    db_begin_transaction();
+    content_rcvid_init("#!fossil /uvlist upload");
+    blob_init(&content, aContent, nContent);
+    unversioned_write(zName, &content, time(0));
+    blob_reset(&content);
+    db_end_transaction(0);
+    CX("<div>Added: %h</div>", zName);
+  }
+  form_begin("enctype='multipart/form-data'", "%R/uvlist");
+  @ <label for='uvupload'>Upload unversioned file:</label>
+  @ <input type='file' id='uvupload' name='f'/>
+  @ <input type='submit' id='uvsubmit' value='Upload' disabled='disabled'/>
+  @ </form>
+  @ <script nonce='%h(style_nonce())'>;/* unversioned.c:%d(__LINE__) */
+  @ var upl = document.getElementById('uvupload');
+  @ var sbm = document.getElementById('uvsubmit');
+  @ upl.onchange = function(){
+  @   if (!upl.value) sbm.setAttribute('disabled', 'disabled');
+  @   else sbm.removeAttribute('disabled');
+  @ }  
+  @ </script>
+}
+
 /*
 ** WEBPAGE: uvlist
 **
@@ -547,6 +605,7 @@ void uvlist_page(void){
   cgi_check_for_malice();
   etag_check(ETAG_DATA,0);
   style_header("Unversioned Files");
+  uvlist_upload();
   if( !db_table_exists("repository","unversioned") ){
     @ No unversioned files on this server
     style_finish_page();
