@@ -134,6 +134,7 @@ set src {
   manifest
   markdown
   markdown_html
+  match
   md5
   merge
   merge3
@@ -211,6 +212,7 @@ set src_ext {
 # be relative to that.
 set extra_files {
   diff.tcl
+  merge.tcl
   markdown.md
   wiki.wiki
   *.js
@@ -240,10 +242,11 @@ set SQLITE_OPTIONS {
   -DSQLITE_MAX_EXPR_DEPTH=0
   -DSQLITE_ENABLE_LOCKING_STYLE=0
   -DSQLITE_DEFAULT_FILE_FORMAT=4
+  -DSQLITE_ENABLE_DBSTAT_VTAB
   -DSQLITE_ENABLE_EXPLAIN_COMMENTS
   -DSQLITE_ENABLE_FTS4
-  -DSQLITE_ENABLE_DBSTAT_VTAB
   -DSQLITE_ENABLE_FTS5
+  -DSQLITE_ENABLE_MATH_FUNCTIONS
   -DSQLITE_ENABLE_STMTVTAB
   -DSQLITE_HAVE_ZLIB
   -DSQLITE_ENABLE_DBPAGE_VTAB
@@ -380,6 +383,7 @@ $(OBJDIR):
 	-mkdir $(OBJDIR)
 
 $(OBJDIR)/translate:	$(SRCDIR_tools)/translate.c
+	-mkdir -p $(OBJDIR)
 	$(XBCC) -o $(OBJDIR)/translate $(SRCDIR_tools)/translate.c
 
 $(OBJDIR)/makeheaders:	$(SRCDIR_tools)/makeheaders.c
@@ -447,7 +451,7 @@ SQLITE3_OBJ   = $(SQLITE3_OBJ.$(SQLITE3_ORIGIN))
 # to 1. If it is set to 0, then there is no need to build or link
 # the linenoise.o object.
 LINENOISE_DEF.0 =
-LINENOISE_DEF.1 = -DHAVE_LINENOISE
+LINENOISE_DEF.1 = -DHAVE_LINENOISE=2
 LINENOISE_DEF.  = $(LINENOISE_DEF.0)
 LINENOISE_OBJ.0 =
 LINENOISE_OBJ.1 = $(OBJDIR)/linenoise.o
@@ -565,14 +569,15 @@ $(OBJDIR)/pikchr.o:	$(SRCDIR_extsrc)/pikchr.c
 $(OBJDIR)/cson_amalgamation.o: $(SRCDIR_extsrc)/cson_amalgamation.c
 	$(XTCC) -c $(SRCDIR_extsrc)/cson_amalgamation.c -o $@
 
-$(SRCDIR_extsrc)/pikchr.js: $(SRCDIR_extsrc)/pikchr.c
+$(SRCDIR_extsrc)/pikchr.js: $(SRCDIR_extsrc)/pikchr.c $(MAKEFILE_LIST)
 	$(EMCC_WRAPPER) -o $@ $(EMCC_OPT) --no-entry <<<NEXT_LINE>>>
-        -sEXPORTED_RUNTIME_METHODS=cwrap,setValue,getValue,stackSave,stackRestore <<<NEXT_LINE>>>
-        -sEXPORTED_FUNCTIONS=_pikchr $(SRCDIR_extsrc)/pikchr.c <<<NEXT_LINE>>>
+        -sEXPORTED_RUNTIME_METHODS=cwrap,ccall,setValue,getValue,stackSave,stackAlloc,stackRestore <<<NEXT_LINE>>>
+        -sEXPORTED_FUNCTIONS=_pikchr,_pikchr_version $(SRCDIR_extsrc)/pikchr.c <<<NEXT_LINE>>>
         -sENVIRONMENT=web <<<NEXT_LINE>>>
         -sMODULARIZE <<<NEXT_LINE>>>
         -sEXPORT_NAME=initPikchrModule <<<NEXT_LINE>>>
         --minify 0
+	$(TCLSH) $(TOPDIR)/tools/randomize-js-names.tcl $(SRCDIR_extsrc)
 	@chmod -x $(SRCDIR_extsrc)/pikchr.wasm
 wasm: $(SRCDIR_extsrc)/pikchr.js
 
