@@ -40,7 +40,7 @@
 ** "test-" or if the command name has a "test" argument, then it becomes
 ** a test command.  If the command name has a "2nd-tier" argument or ends
 ** with a "*" character, it is second tier.  If the command name has an "alias"
-** argument or ends with a "#" character, it is an alias: another name  
+** argument or ends with a "#" character, it is an alias: another name
 ** (a one-to-one replacement) for a command.  Examples:
 **
 **        COMMAND:  abcde*
@@ -87,21 +87,22 @@
 ** These macros must match similar macros in dispatch.c.
 **
 ** Allowed values for CmdOrPage.eCmdFlags. */
-#define CMDFLAG_1ST_TIER     0x0001     /* Most important commands */
-#define CMDFLAG_2ND_TIER     0x0002     /* Obscure and seldom used commands */
-#define CMDFLAG_TEST         0x0004     /* Commands for testing only */
-#define CMDFLAG_WEBPAGE      0x0008     /* Web pages */
-#define CMDFLAG_COMMAND      0x0010     /* A command */
-#define CMDFLAG_SETTING      0x0020     /* A setting */
-#define CMDFLAG_VERSIONABLE  0x0040     /* A versionable setting */
-#define CMDFLAG_BLOCKTEXT    0x0080     /* Multi-line text setting */
-#define CMDFLAG_BOOLEAN      0x0100     /* A boolean setting */
-#define CMDFLAG_RAWCONTENT   0x0200     /* Do not interpret webpage content */
-#define CMDFLAG_SENSITIVE    0x0400     /* Security-sensitive setting */
-#define CMDFLAG_HIDDEN       0x0800     /* Elide from most listings */
-#define CMDFLAG_LDAVG_EXEMPT 0x1000     /* Exempt from load_control() */
-#define CMDFLAG_ALIAS        0x2000     /* Command aliases */
-#define CMDFLAG_KEEPEMPTY    0x4000     /* Do not unset empty settings */
+#define CMDFLAG_1ST_TIER     0x00001    /* Most important commands */
+#define CMDFLAG_2ND_TIER     0x00002    /* Obscure and seldom used commands */
+#define CMDFLAG_TEST         0x00004    /* Commands for testing only */
+#define CMDFLAG_WEBPAGE      0x00008    /* Web pages */
+#define CMDFLAG_COMMAND      0x00010    /* A command */
+#define CMDFLAG_SETTING      0x00020    /* A setting */
+#define CMDFLAG_VERSIONABLE  0x00040    /* A versionable setting */
+#define CMDFLAG_BLOCKTEXT    0x00080    /* Multi-line text setting */
+#define CMDFLAG_BOOLEAN      0x00100    /* A boolean setting */
+#define CMDFLAG_RAWCONTENT   0x00200    /* Do not interpret webpage content */
+#define CMDFLAG_SENSITIVE    0x00400    /* Security-sensitive setting */
+#define CMDFLAG_HIDDEN       0x00800    /* Elide from most listings */
+#define CMDFLAG_LDAVG_EXEMPT 0x01000    /* Exempt from load_control() */
+#define CMDFLAG_ALIAS        0x02000    /* Command aliases */
+#define CMDFLAG_KEEPEMPTY    0x04000    /* Do not unset empty settings */
+#define CMDFLAG_ABBREVSUBCMD 0x08000    /* Abbreviated subcmd in help text */
 /**************************************************************************/
 
 /*
@@ -282,6 +283,9 @@ void scan_for_label(const char *zLabel, char *zLine, int eType){
       aEntry[nUsed].eType |= CMDFLAG_HIDDEN;
     }else if( j==14 && strncmp(&zLine[i], "loadavg-exempt", 14)==0 ){
       aEntry[nUsed].eType |= CMDFLAG_LDAVG_EXEMPT;
+    }else if( (j==23 && strncmp(&zLine[i], "abbreviated-subcommands", 23)==0)
+           || (j==12 && strncmp(&zLine[i], "abbrv-subcom", 12)==0) ){
+      aEntry[nUsed].eType |= CMDFLAG_ABBREVSUBCMD;
     }else{
       fprintf(stderr, "%s:%d: unknown option: '%.*s'\n",
               zFile, nLine, j, &zLine[i]);
@@ -330,6 +334,11 @@ void scan_for_default(const char *zLine){
   aEntry[nUsed-1].zDflt = string_dup(z,len);
 }
 
+/* Local strcpy() clone to squelch an unwarranted warning from OpenBSD. */
+static void local_strcpy(char *dest, const char *src){
+  while( (*(dest++) = *(src++))!=0 ){}
+}
+
 /*
 ** Scan a line for a function that implements a web page or command.
 */
@@ -351,7 +360,7 @@ void scan_for_func(char *zLine){
       zHelp[nHelp++] = '\n';
     }else{
       if( strncmp(&zLine[3], "Usage: ", 6)==0 ) nHelp = 0;
-      strcpy(&zHelp[nHelp], &zLine[3]);
+      local_strcpy(&zHelp[nHelp], &zLine[3]);
       nHelp += strlen(&zHelp[nHelp]);
     }
     return;
